import os

import pytest

from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import SELinuxCustom, SELinuxFacts, SELinuxModule, SELinuxModules, SELinuxRequestRPMs
from leapp.reporting import Report

TEST_MODULES = [
    ["400", "mock1"],
    ["99", "mock1"],
    ["200", "mock1"],
    ["400", "mock2"],
    ["999", "mock3"],
    ["400", "permissive_abrt_t"]
]

TEST_TEMPLATES = [
    ["200", "base_container"],
    ["200", "home_container"],
]

# [0] will be passed to the actor as "removed"
# [1] will not be passed to the actor and should not be removed
# the rest will be passed as valid and should be applied by the actor
# [4]-[7] cannot be added without tweaking the commands (testing the fix for
# "semanage export" bug where "-a" is exported instead of "-m")
SEMANAGE_COMMANDS = [
    ['fcontext', '-t', 'cgdcbxd_var_run_t', "'/ganesha(/.*)?'"],
    ['user', 'yolo', '-R', 'user_r'],
    ['fcontext', '-t', 'httpd_sys_content_t', "'/web(/.*)?'"],
    ['port', '-t', 'http_port_t', '-p', 'udp', '81'],
    ['port', '-t', 'ssh_port_t', '-p', 'tcp', '8021'],
    ['user', 'user_u', '-R', 'user_r', '-R', 'staff_r'],
    ['login', '-s', 'guest_u', '__default__', '-r', 's0'],
    ['fcontext', '-t', 'httpd_sys_content_t', "'/vmlinuz.*'", '-f', 'l']
]


def _run_cmd(cmd, logmsg="", split=True):
    try:
        return run(cmd, split=split).get("stdout", "")
    except CalledProcessError as e:
        if logmsg:
            api.current_logger().warning("{}: {}".format(logmsg, e.stderr))
    return None


def find_module_semodule(semodule_lfull, name, priority):
    return next((line for line in semodule_lfull if (name in line and priority in line)), None)


def find_semanage_rule(rules, rule):
    return next((r for r in rules if all(word in r for word in rule)), None)


@pytest.fixture(scope="function")
def destructive_selinux_env():
    # apply SEMANAGE_COMMANDS[1] so that we can test that the actor did not remove it
    _run_cmd(["semanage", SEMANAGE_COMMANDS[1][0], "-a"] + SEMANAGE_COMMANDS[1][1:],
             "Error applying selinux customizations before test")

    yield

    semodule_command = ["semodule"]
    for priority, module in TEST_MODULES:
        semodule_command.extend(["-X", priority, "-r", module])
    _run_cmd(semodule_command, "Error removing modules after testing!")

    for command in SEMANAGE_COMMANDS:
        _run_cmd(["semanage", command[0], "-d"] + [x.strip('"\'') for x in command[1:]],
                 "Failed to remove SELinux customizations after testing")


@pytest.mark.skipif(os.getenv("DESTRUCTIVE_TESTING", False) in [False, "0"],
                    reason='Test disabled by default because it would modify the system')
def test_SELinuxApplyCustom(current_actor_context, destructive_selinux_teardown):

    semodule_list = [SELinuxModule(name=module, priority=int(prio),
                                   content="(allow domain proc_type (file (getattr open read)))", removed=[])
                     for (prio, module) in TEST_MODULES]
    template_list = [SELinuxModule(name=module, priority=int(prio),
                                   content="", removed=[])
                     for (prio, module) in TEST_TEMPLATES]

    commands = [" ".join([c[0], "-a"] + c[1:]) for c in SEMANAGE_COMMANDS[2:]]
    semanage_removed = [" ".join([SEMANAGE_COMMANDS[0][0], "-a"] + SEMANAGE_COMMANDS[0][1:])]

    current_actor_context.feed(SELinuxModules(modules=semodule_list, templates=template_list))
    current_actor_context.feed(SELinuxCustom(commands=commands, removed=semanage_removed))
    current_actor_context.run()

    semodule_lfull = _run_cmd(["semodule", "-lfull"],
                              "Error listing selinux modules")
    semanage_export = _run_cmd(["semanage", "export"],
                               "Error listing selinux customizations")

    # check that all reported modules where introduced to the system
    for priority, name in TEST_MODULES + TEST_TEMPLATES:
        if priority not in ('100', '200'):
            assert find_module_semodule(semodule_lfull, name, priority)
    # check that all valid commands where introduced to the system (SEMANAGE_COMMANDS[2:])
    # and that SEMANAGE_COMMANDS[1] was not removed
    for command in SEMANAGE_COMMANDS[1:-1]:
        assert find_semanage_rule(semanage_export, command)



from leapp.libraries.common.config import architecture
from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import RoceDetected


def get_roce_nics_lines():
    """
    Return basic info about RoCE NICs using nmcli

    When RoCE is configured on the system, we should find Mellanox a device
    in the `nmcli` output, which is always specified after status line
    of an interface, e.g.:
        # nmcli
        ens1765: connected to ens1765
            "Mellanox MT27710"
            ethernet (mlx5_core), 82:28:9B:1B:28:2C, hw, mtu 1500
            inet4 192.168.0.1/16
            route4 192.168.0.1/16
            inet6 fe80::d8c5:3a67:1abb:dcca/64
            route6 fe80::/64
    In this case, the function returns the list of lines with RoCE NICs.
    So for the example above:
        ['ens1765: connected to ens1765']

    NOTE: It is unexpected that a NIC itself could contain a 'mellanox'
    substring. In such a case additional unexpected lines could be returned.
    However, as we are interested only about lines with 'connected to' and 'connecting'
    substrings, we know we will filter out any invalid lines later, so it's
    no problem for us.
    """
    # nmcli | grep --no-group-separator -B1 -i "mellanox" | sed -n 1~2p
    roce_nic_lines = []
    try:
        nmcli_output = run(['nmcli'], split=True)['stdout']
    except (CalledProcessError, OSError) as e:
        # this is theoretical
        # If the command fails, most likely the network is not configured
        # or it is not configured in a 'supported' way - definitely not
        # for RoCE.
        api.current_logger().warning(
            'Cannot examine network connections via NetworkManager.'
            ' Assuming RoCE is not present. Detail: {}'.format(str(e))
        )
        return roce_nic_lines

    for i, line in enumerate(nmcli_output):
        if 'mellanox' in line.lower() and i > 0:
            roce_nic_lines.append(nmcli_output[i-1].strip())
    return roce_nic_lines


def _parse_NIC(nmcli_line):
    return nmcli_line.split(':')[0]


def process():
    if not architecture.matches_architecture(architecture.ARCH_S390X):
        # The check is valid only on S390X architecture
        return
    connected_nics = []
    connecting_nics = []
    for line in get_roce_nics_lines():
        if 'connected to' in line:
            connected_nics.append(_parse_NIC(line))
        elif 'connecting' in line:
            connecting_nics.append(_parse_NIC(line))
    if connected_nics or connecting_nics:
        api.produce(RoceDetected(
           roce_nics_connected=connected_nics,
           roce_nics_connecting=connecting_nics,
        ))



from leapp.actors import Actor
from leapp.libraries.actor import scan_target_os_iso
from leapp.models import CustomTargetRepository, TargetOSInstallationImage
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class ScanTargetISO(Actor):
    """Scans the provided target OS ISO image to use as a content source for the IPU, if any."""

    name = 'scan_target_os_image'
    consumes = ()
    produces = (CustomTargetRepository, TargetOSInstallationImage,)
    tags = (IPUWorkflowTag, FactsPhaseTag)

    def process(self):
        scan_target_os_iso.inform_ipu_about_request_to_use_target_iso()



import mock
import pytest

from leapp.cli.commands import command_utils
from leapp.exceptions import CommandError


@mock.patch("leapp.cli.commands.command_utils.get_upgrade_paths_config",
            return_value={"default": {"7.9": ["8.4"], "8.6": ["9.0"], "7": ["8.4"], "8": ["9.0"]}})
def test_get_target_version(mock_open, monkeypatch):

    monkeypatch.setattr(command_utils, 'get_os_release_version_id', lambda x: '8.6')
    assert command_utils.get_target_version('default') == '9.0'

    monkeypatch.setenv('LEAPP_DEVEL_TARGET_RELEASE', '')
    monkeypatch.setattr(command_utils, 'get_os_release_version_id', lambda x: '8.6')
    assert command_utils.get_target_version('default') == '9.0'

    monkeypatch.delenv('LEAPP_DEVEL_TARGET_RELEASE', raising=True)
    # unsupported path
    monkeypatch.setattr(command_utils, 'get_os_release_version_id', lambda x: '8.5')
    assert command_utils.get_target_version('default') == '9.0'


@mock.patch("leapp.cli.commands.command_utils.get_upgrade_paths_config",
            return_value={"default": {"7.9": ["8.4"], "8.6": ["9.0"], "7": ["8.4"], "8": ["9.0"]}})
def test_vet_upgrade_path(mock_open, monkeypatch):
    monkeypatch.setattr(command_utils, 'get_os_release_version_id', lambda x: '8.6')

    # make sure env var LEAPP_DEVEL_TARGET_RELEASE takes precedence
    # when env var set to a bad version - abort the upgrade
    args = mock.Mock(target='9.0')
    monkeypatch.setenv('LEAPP_DEVEL_TARGET_RELEASE', '1.2badsemver')
    with pytest.raises(CommandError) as err:
        command_utils.vet_upgrade_path(args)
        assert 'Unexpected format of target version' in err
    # MAJOR.MINOR.PATCH is considered as bad version, only MAJOR.MINOR is accepted
    args = mock.Mock(target='9.0')
    monkeypatch.setenv('LEAPP_DEVEL_TARGET_RELEASE', '9.0.0')
    with pytest.raises(CommandError) as err:
        command_utils.vet_upgrade_path(args)
        assert 'Unexpected format of target version' in err
    # when env var set to a version not in upgrade_paths map - go on and use it
    monkeypatch.setenv('LEAPP_DEVEL_TARGET_RELEASE', '1.2')
    assert command_utils.vet_upgrade_path(args) == ('1.2', 'default')
    # no env var set, --target is set to proper version
    monkeypatch.delenv('LEAPP_DEVEL_TARGET_RELEASE', raising=False)
    assert command_utils.vet_upgrade_path(args) == ('9.0', 'default')
    # env var is set to proper version, --target is set to a bad one - use env var and go on with the upgrade
    monkeypatch.setenv('LEAPP_DEVEL_TARGET_RELEASE', '9.0')
    args = mock.Mock(target='1.2')
    assert command_utils.vet_upgrade_path(args) == ('9.0', 'default')



from leapp.actors import Actor
from leapp.libraries.actor import ipuworkflowconfig
from leapp.models import IPUConfig
from leapp.reporting import Report
from leapp.tags import IPUWorkflowTag


class IPUWorkflowConfig(Actor):
    """
    IPU workflow config actor
    """

    name = 'ipu_workflow_config'
    consumes = ()
    produces = (IPUConfig, Report)
    tags = (IPUWorkflowTag,)

    def process(self):
        ipuworkflowconfig.produce_ipu_config(self)



import os

from leapp.libraries.stdlib import api
from leapp.models import CopyFile, TargetUserSpacePreupgradeTasks


def process():
    src = "/etc/dnf/dnf.conf"
    if os.path.exists("/etc/leapp/files/dnf.conf"):
        src = "/etc/leapp/files/dnf.conf"

    api.current_logger().debug(
        "Copying dnf.conf at {} to the target userspace".format(src)
    )
    api.produce(
        TargetUserSpacePreupgradeTasks(
            copy_files=[CopyFile(src=src, dst="/etc/dnf/dnf.conf")]
        )
    )



import itertools

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.common import kernel as kernel_lib
from leapp.libraries.common.config.version import get_source_version
from leapp.libraries.stdlib import api
from leapp.models import DistributionSignedRPM, KernelInfo


def scan_source_kernel():
    uname_r = api.current_actor().configuration.kernel
    installed_rpms = [msg.items for msg in api.consume(DistributionSignedRPM)]
    installed_rpms = list(itertools.chain(*installed_rpms))

    kernel_type = kernel_lib.determine_kernel_type_from_uname(get_source_version(), uname_r)
    kernel_pkg_info = kernel_lib.get_kernel_pkg_info_for_uname_r(uname_r)

    kernel_pkg_id = (kernel_pkg_info.name, kernel_pkg_info.version, kernel_pkg_info.release, kernel_pkg_info.arch)
    kernel_pkg = None
    for pkg in installed_rpms:
        pkg_id = (pkg.name, pkg.version, pkg.release, pkg.arch)
        if kernel_pkg_id == pkg_id:
            kernel_pkg = pkg
            break

    if not kernel_pkg:
        raise StopActorExecutionError(message='Unable to identify package providing the booted kernel.')

    kernel_info = KernelInfo(pkg=kernel_pkg, type=kernel_type, uname_r=uname_r)
    api.produce(kernel_info)



from leapp.libraries.actor import spamassassinconfigcheck
from leapp.libraries.common.testutils import create_report_mocked
from leapp.models import SpamassassinFacts


def test_check_spamc_config_tlsv1():
    facts = SpamassassinFacts(spamc_ssl_argument='tlsv1', service_overriden=False)
    report_func = create_report_mocked()

    spamassassinconfigcheck._check_spamc_config(facts, report_func)

    assert report_func.called == 1
    report_fields = report_func.report_fields
    assert 'specifying the TLS version' in report_fields['title']
    assert 'SSLv3' in report_fields['title']
    assert '--ssl' in report_fields['summary']
    assert 'SSLv3' in report_fields['summary']
    assert 'spamc configuration file' in report_fields['summary']
    assert '--ssl tlsv1' in report_fields['summary']
    assert all('update your scripts' in r['context'] for r in report_fields['detail']['remediations'])
    assert report_fields['severity'] == 'medium'


def test_check_spamc_config_sslv3():
    facts = SpamassassinFacts(spamc_ssl_argument='sslv3', service_overriden=False)
    report_func = create_report_mocked()

    spamassassinconfigcheck._check_spamc_config(facts, report_func)

    assert report_func.called == 1
    report_fields = report_func.report_fields
    assert 'specifying the TLS version' in report_fields['title']
    assert 'SSLv3' in report_fields['title']
    assert '--ssl' in report_fields['summary']
    assert 'SSLv3' in report_fields['summary']
    assert 'spamc configuration file' in report_fields['summary']
    assert '--ssl sslv3' in report_fields['summary']
    assert all('update your scripts' in r['context'] for r in report_fields['detail']['remediations'])
    assert report_fields['severity'] == 'high'


def test_check_spamc_config_correct_config():
    facts = SpamassassinFacts(spamc_ssl_argument=None, service_overriden=False)
    report_func = create_report_mocked()

    spamassassinconfigcheck._check_spamc_config(facts, report_func)

    assert report_func.called == 1
    report_fields = report_func.report_fields
    assert 'specifying the TLS version' in report_fields['title']
    assert 'SSLv3' in report_fields['title']
    assert '--ssl' in report_fields['summary']
    assert 'SSLv3' in report_fields['summary']
    assert 'spamc configuration file' not in report_fields['summary']
    assert all('update your scripts' in r['context'] for r in report_fields['detail']['remediations'])
    assert report_fields['severity'] == 'medium'


def test_check_spamd_config_ssl_tlsv1():
    facts = SpamassassinFacts(spamd_ssl_version='tlsv1', service_overriden=False)
    report_func = create_report_mocked()

    spamassassinconfigcheck._check_spamd_config_ssl(facts, report_func)

    assert report_func.called == 1
    report_fields = report_func.report_fields
    assert 'specifying the TLS version' in report_fields['title']
    assert 'SSLv3' in report_fields['title']
    assert '--ssl-version' in report_fields['summary']
    assert 'SSLv3' in report_fields['summary']
    assert 'sysconfig' in report_fields['summary']
    assert '--ssl-version tlsv1' in report_fields['summary']
    assert all('update your scripts' in r['context'] for r in report_fields['detail']['remediations'])
    assert report_fields['severity'] == 'medium'


def test_check_spamd_config_ssl_sslv3():
    facts = SpamassassinFacts(spamd_ssl_version='sslv3', service_overriden=False)
    report_func = create_report_mocked()

    spamassassinconfigcheck._check_spamd_config_ssl(facts, report_func)

    assert report_func.called == 1
    report_fields = report_func.report_fields
    assert 'specifying the TLS version' in report_fields['title']
    assert 'SSLv3' in report_fields['title']
    assert '--ssl-version' in report_fields['summary']
    assert 'SSLv3' in report_fields['summary']
    assert 'sysconfig' in report_fields['summary']
    assert '--ssl-version sslv3' in report_fields['summary']
    assert all('update your scripts' in r['context'] for r in report_fields['detail']['remediations'])
    assert report_fields['severity'] == 'high'


def test_check_spamd_config_ssl_correct_config():
    facts = SpamassassinFacts(spamd_ssl_version=None, service_overriden=False)
    report_func = create_report_mocked()

    spamassassinconfigcheck._check_spamd_config_ssl(facts, report_func)

    assert report_func.called == 1
    report_fields = report_func.report_fields
    assert 'specifying the TLS version' in report_fields['title']
    assert 'SSLv3' in report_fields['title']
    assert '--ssl-version' in report_fields['summary']
    assert 'SSLv3' in report_fields['summary']
    assert 'sysconfig' not in report_fields['summary']
    assert all('update your scripts' in r['context'] for r in report_fields['detail']['remediations'])
    assert report_fields['severity'] == 'medium'


def test_check_spamd_config_service_type_service_overriden():
    facts = SpamassassinFacts(service_overriden=True)
    report_func = create_report_mocked()

    spamassassinconfigcheck._check_spamd_config_service_type(facts, report_func)

    assert report_func.called == 1
    report_fields = report_func.report_fields
    assert 'type of the spamassassin systemd service' in report_fields['title']
    assert 'The type of spamassassin.service' in report_fields['summary']
    assert 'overridden' in report_fields['summary']
    assert report_fields['severity'] == 'medium'


def test_check_spamd_config_service_type_service_not_overriden():
    facts = SpamassassinFacts(service_overriden=False)
    report_func = create_report_mocked()

    spamassassinconfigcheck._check_spamd_config_service_type(facts, report_func)

    assert report_func.called == 1
    report_fields = report_func.report_fields
    assert 'type of the spamassassin systemd service' in report_fields['title']
    assert 'The type of spamassassin.service' in report_fields['summary']
    assert 'will be updated' in report_fields['summary']
    assert report_fields['severity'] == 'medium'


def test_report_sa_update_change():
    report_func = create_report_mocked()

    spamassassinconfigcheck._report_sa_update_change(report_func)

    assert report_func.called == 1
    report_fields = report_func.report_fields
    assert 'sa-update no longer supports SHA1' in report_fields['title']
    assert 'no longer supports SHA1' in report_fields['summary']
    assert 'SHA256/SHA512' in report_fields['summary']
    assert '--channel or --channelfile' in report_fields['summary']
    assert '--install' in report_fields['summary']
    assert report_fields['severity'] == 'low'



from leapp import reporting
from leapp.libraries.stdlib import api
from leapp.models import CryptoPolicyInfo, InstalledRPM

FMT_LIST_SEPARATOR = '\n    - '

# FIXME(pstodulk): Adding the links to the summary as information inside has
# serious impact. This will create a duplication of links for Satellite and
# Cockpit UI when reading the report, but we need to be sure they are printed
# in the /var/log/leapp/leapp-report.txt file as well. Remove them when the
# framework prints external links in the file as well.
SUMMARY_FMT = (
    'Digital signatures using SHA-1 hash algorithm are no longer considered'
    ' secure and are not allowed to be used on RHEL 9 systems by default.'
    ' This causes issues when using DNF/RPM to handle packages with RSA/SHA1'
    ' signatures as the signature cannot be checked with the default'
    ' cryptographic policy. Any such packages cannot be installed, removed,'
    ' or replaced unless the signature check is disabled in dnf/rpm'
    ' or SHA-1 is enabled using non-default crypto-policies.'
    ' For more information see the following documents:\n'
    '  - Major changes in RHEL 9: {major_changes_url}\n'
    '  - Security Considerations in adopting RHEL 9: {crypto_policies_url}\n'
    ' The list of problematic packages: {bad_pkgs}'
)

REMEDY_HINT = (
    'It is recommended that you contact your package vendor and ask them for new'
    ' builds signed with supported signatures and install the new packages before'
    ' the upgrade. If this is not possible you may instead'
    ' remove the incompatible packages.'
)

MAJOR_CHANGE_URL = 'https://red.ht/rhel-9-overview-major-changes'
CRYPTO_POLICIES_URL = 'https://red.ht/rhel-9-security-considerations'


def _get_rpms_with_sha1_sig():
    installed_rpms = next(api.consume(InstalledRPM)).items
    return [pkg for pkg in installed_rpms if 'SHA1,' in pkg.pgpsig]


def _is_sha1_allowed(current_policy):
    """
    Return True if we are sure the current policy allows SHA-1 on RHEL 9. False otherwise

    The LEGACY policy and policies like DEFAULT:SHA1 enables SHA-1 on RHEL 9.
    """
    # TODO(pstodulk): this is just naive implementation
    # NOTE: The SHA1 sub policy does not exist on RHEL 8. It has to be created
    # manually by user in such a case.
    # NOTE: for now, limit the check for :SHA1 to DEFAULT:SHA1 only as otherwise we will
    # not be probably able to set correctly the policies inside the target
    # userspace container
    if current_policy == 'LEGACY' or "DEFAULT:SHA1" in current_policy:
        return True
    return False


def process():
    # TODO(pstodulk): add link to the official announce of the change in crypto policies
    bad_rpms = _get_rpms_with_sha1_sig()
    cpi = next(api.consume(CryptoPolicyInfo), None)
    if bad_rpms:
        bad_rpms_str = ''.join([
            '{prefix}{pkgname} ({sig})'.format(prefix=FMT_LIST_SEPARATOR, pkgname=pkg.name, sig=pkg.pgpsig)
            for pkg in bad_rpms
        ])
        report = [
            reporting.Title('Detected RPMs with RSA/SHA1 signature'),
            reporting.Summary(SUMMARY_FMT.format(
                major_changes_url=MAJOR_CHANGE_URL,
                crypto_policies_url=CRYPTO_POLICIES_URL,
                bad_pkgs=bad_rpms_str
            )),
            reporting.Severity(reporting.Severity.HIGH),
            reporting.Groups([reporting.Groups.SECURITY, reporting.Groups.SANITY]),
            reporting.Remediation(hint=REMEDY_HINT),
            reporting.ExternalLink(
                url=MAJOR_CHANGE_URL,
                title='Major changes in RHEL 9'
            ),
            reporting.ExternalLink(
                url=CRYPTO_POLICIES_URL,
                title='Security Considerations in adopting RHEL 9'
            )
        ]
        if not _is_sha1_allowed(cpi.current_policy):
            report.append(reporting.Groups([reporting.Groups.INHIBITOR]))
        reporting.create_report(report)



import textwrap

from leapp import reporting


def satellite_upgrade_check(facts):
    if facts.postgresql.local_postgresql:
        if facts.postgresql.old_var_lib_pgsql_data:
            title = "Old PostgreSQL data found in /var/lib/pgsql/data"
            summary = """
            The upgrade wants to move PostgreSQL data to /var/lib/pgsql/data,
            but this directory already exists on your system.
            Please make sure /var/lib/pgsql/data doesn't exist prior to the upgrade.
            """
            reporting.create_report([
                reporting.Title(title),
                reporting.Summary(textwrap.dedent(summary).strip()),
                reporting.Severity(reporting.Severity.HIGH),
                reporting.Groups([]),
                reporting.Groups([reporting.Groups.INHIBITOR])
            ])

        title = "Satellite PostgreSQL data migration"
        flags = []
        severity = reporting.Severity.MEDIUM
        reindex_msg = textwrap.dedent("""
        After the data has been moved to the new location, all databases will require a REINDEX.
        This will happen automatically during the first boot of the system.
        """).strip()

        if facts.postgresql.same_partition:
            migration_msg = "Your PostgreSQL data will be automatically migrated."
        else:
            scl_psql_path = '/var/opt/rh/rh-postgresql12/lib/pgsql/data/'
            if facts.postgresql.space_required > facts.postgresql.space_available:
                storage_message = """You currently don't have enough free storage to move the data.
                Automatic moving cannot be performed."""
                flags = [reporting.Groups.INHIBITOR]
                severity = reporting.Severity.HIGH
            else:
                storage_message = """You currently have enough free storage to move the data.
                This operation can be performed by the upgrade process."""
            migration_msg = """
            Your PostgreSQL data in {} is currently on a dedicated volume.
            PostgreSQL on RHEL8 expects the data to live in /var/lib/pgsql/data.
            {}
            However, instead of moving the data over, you might want to consider manually adapting your mounts,
            so that the contents of {} are available in /var/lib/pgsql/data.
            """.format(scl_psql_path, storage_message, scl_psql_path)

        summary = "{}\n{}".format(textwrap.dedent(migration_msg).strip(), reindex_msg)

        reporting.create_report([
            reporting.Title(title),
            reporting.Summary(summary),
            reporting.Severity(severity),
            reporting.Groups([]),
            reporting.Groups(flags)
        ])



import os
from functools import partial

import pytest

from leapp.libraries.common import systemd
from leapp.libraries.common.testutils import logger_mocked
from leapp.libraries.stdlib import api
from leapp.models import SystemdServiceFile, SystemdServicePreset

CURR_DIR = os.path.dirname(os.path.abspath(__file__))


def test_get_service_files(monkeypatch):
    def run_mocked(cmd, *args, **kwargs):
        if cmd == ['systemctl', 'list-unit-files'] + systemd._SYSTEMCTL_CMD_OPTIONS:
            return {'stdout': [
                'auditd.service                                enabled',
                'crond.service                                 enabled ',
                'dbus.service                                  static  ',
                'dnf-makecache.service                         static  ',
                'firewalld.service                             enabled ',
                'getty@.service                                enabled ',
                'gssproxy.service                              disabled',
                'kdump.service                                 enabled ',
                'mdmon@.service                                static  ',
                'nfs.service                                   disabled',
                'polkit.service                                static  ',
                'rescue.service                                static  ',
                'rngd.service                                  enabled ',
                'rsyncd.service                                disabled',
                'rsyncd@.service                               static  ',
                'smartd.service                                enabled ',
                'sshd.service                                  enabled ',
                'sshd@.service                                 static  ',
                'wpa_supplicant.service                        disabled'
            ]}
        raise ValueError('Attempted to call unexpected command: {}'.format(cmd))

    monkeypatch.setattr(systemd, 'run', run_mocked)
    service_files = systemd.get_service_files()

    expected = [
        SystemdServiceFile(name='auditd.service', state='enabled'),
        SystemdServiceFile(name='crond.service', state='enabled'),
        SystemdServiceFile(name='dbus.service', state='static'),
        SystemdServiceFile(name='dnf-makecache.service', state='static'),
        SystemdServiceFile(name='firewalld.service', state='enabled'),
        SystemdServiceFile(name='getty@.service', state='enabled'),
        SystemdServiceFile(name='gssproxy.service', state='disabled'),
        SystemdServiceFile(name='kdump.service', state='enabled'),
        SystemdServiceFile(name='mdmon@.service', state='static'),
        SystemdServiceFile(name='nfs.service', state='disabled'),
        SystemdServiceFile(name='polkit.service', state='static'),
        SystemdServiceFile(name='rescue.service', state='static'),
        SystemdServiceFile(name='rngd.service', state='enabled'),
        SystemdServiceFile(name='rsyncd.service', state='disabled'),
        SystemdServiceFile(name='rsyncd@.service', state='static'),
        SystemdServiceFile(name='smartd.service', state='enabled'),
        SystemdServiceFile(name='sshd.service', state='enabled'),
        SystemdServiceFile(name='sshd@.service', state='static'),
        SystemdServiceFile(name='wpa_supplicant.service', state='disabled')
    ]

    assert service_files == expected


def test_preset_files_overrides():
    etc_files = [
        '/etc/systemd/system-preset/00-abc.preset',
        '/etc/systemd/system-preset/preset_without_prio.preset'
    ]
    usr_files = [
        '/usr/lib/systemd/system-preset/00-abc.preset',
        '/usr/lib/systemd/system-preset/99-xyz.preset',
        '/usr/lib/systemd/system-preset/preset_without_prio.preset'
    ]

    expected = [
        '/usr/lib/systemd/system-preset/99-xyz.preset',
        '/etc/systemd/system-preset/00-abc.preset',
        '/etc/systemd/system-preset/preset_without_prio.preset'
    ]

    presets = systemd._join_presets_resolving_overrides(etc_files, usr_files)
    assert sorted(presets) == sorted(expected)


def test_preset_files_block_override(monkeypatch):
    etc_files = [
        '/etc/systemd/system-preset/00-abc.preset'
    ]
    usr_files = [
        '/usr/lib/systemd/system-preset/00-abc.preset',
        '/usr/lib/systemd/system-preset/99-xyz.preset'
    ]

    expected = [
        '/usr/lib/systemd/system-preset/99-xyz.preset',
    ]

    def islink_mocked(path):
        return path == '/etc/systemd/system-preset/00-abc.preset'

    def readlink_mocked(path):
        if path == '/etc/systemd/system-preset/00-abc.preset':
            return '/dev/null'
        raise OSError

    monkeypatch.setattr(os.path, 'islink', islink_mocked)
    monkeypatch.setattr(os, 'readlink', readlink_mocked)

    presets = systemd._join_presets_resolving_overrides(etc_files, usr_files)
    assert sorted(presets) == sorted(expected)


TEST_SYSTEMD_LOAD_PATH = [os.path.join(CURR_DIR, 'test_systemd_files/')]

TESTING_PRESET_FILES = [
    os.path.join(CURR_DIR, '00-test.preset'),
    os.path.join(CURR_DIR, '01-test.preset')
]

TESTING_PRESET_WITH_INVALID_ENTRIES = os.path.join(CURR_DIR, '05-invalid.preset')

_PARSE_PRESET_ENTRIES_TEST_DEFINITION = (
    ('enable example.service', {'example.service': 'enable'}),
    ('disable abc.service', {'abc.service': 'disable'}),
    ('enable template@.service', {'template@.service': 'enable'}),
    ('disable template2@.service', {'template2@.service': 'disable'}),
    ('disable template@.service instance1 instance2', {
        'template@instance1.service': 'disable',
        'template@instance2.service': 'disable'
    }),
    ('enable globbed*.service', {'globbed-one.service': 'enable', 'globbed-two.service': 'enable'}),
    ('enable example.*', {'example.service': 'enable', 'example.socket': 'enable'}),
    ('disable *', {
            'example.service': 'disable',
            'abc.service': 'disable',
            'template@.service': 'disable',
            'template2@.service': 'disable',
            'globbed-one.service': 'disable',
            'globbed-two.service': 'disable',
            'example.socket': 'disable',
            'extra.service': 'disable'
    })
)


@pytest.mark.parametrize('entry,expected', _PARSE_PRESET_ENTRIES_TEST_DEFINITION)
def test_parse_preset_entry(monkeypatch, entry, expected):
    presets = {}
    systemd._parse_preset_entry(entry, presets, TEST_SYSTEMD_LOAD_PATH)
    assert presets == expected


@pytest.mark.parametrize(
    'entry',
    [
        ('hello.service'),
        ('mask hello.service'),
        ('enable'),
    ]
)
def test_parse_preset_entry_invalid(monkeypatch, entry):
    presets = {}
    with pytest.raises(ValueError, match=r'^Invalid preset file entry: '):
        systemd._parse_preset_entry(entry, presets, TEST_SYSTEMD_LOAD_PATH)


def test_parse_preset_files(monkeypatch):

    expected = {
        'example.service': 'enable',
        'example.socket': 'disable',
        'abc.service': 'disable',
        'template@.service': 'disable',
        'template@instance1.service': 'enable',
        'template@instance2.service': 'enable',
        'globbed-one.service': 'enable',
        'globbed-two.service': 'enable',
        'extra.service': 'disable',
        'template2@.service': 'disable'
    }

    presets = systemd._parse_preset_files(TESTING_PRESET_FILES, TEST_SYSTEMD_LOAD_PATH, False)
    assert presets == expected


def test_parse_preset_files_invalid():
    with pytest.raises(ValueError):
        systemd._parse_preset_files(
            [TESTING_PRESET_WITH_INVALID_ENTRIES], TEST_SYSTEMD_LOAD_PATH, ignore_invalid_entries=False
        )


def test_parse_preset_files_ignore_invalid(monkeypatch):
    monkeypatch.setattr(api, 'current_logger', logger_mocked())

    invalid_preset_files = [TESTING_PRESET_WITH_INVALID_ENTRIES]
    presets = systemd._parse_preset_files(
        invalid_preset_files, TEST_SYSTEMD_LOAD_PATH, ignore_invalid_entries=True
    )

    for entry in ('enable', 'hello.service', 'mask hello.service'):
        msg = 'Invalid preset file {}: Invalid preset file entry: "{}"'.format(invalid_preset_files[0], entry)
        assert msg in api.current_logger.warnmsg

    assert presets == {'example.service': 'disable'}


def parse_preset_files_mocked():
    mocked = partial(systemd._parse_preset_files, load_path=TEST_SYSTEMD_LOAD_PATH)

    def impl(preset_files, load_path, ignore_invalid_entries):
        return mocked(preset_files, ignore_invalid_entries=ignore_invalid_entries)
    return impl


def test_get_service_preset_files(monkeypatch):

    def get_system_preset_files_mocked():
        return TESTING_PRESET_FILES

    monkeypatch.setattr(systemd, '_get_system_preset_files', get_system_preset_files_mocked)
    monkeypatch.setattr(systemd, '_parse_preset_files', parse_preset_files_mocked())

    service_files = [
        SystemdServiceFile(name='abc.service', state='transient'),
        SystemdServiceFile(name='example.service', state='static'),
        SystemdServiceFile(name='example.socket', state='masked'),
        SystemdServiceFile(name='extra.service', state='disabled'),
        SystemdServiceFile(name='template2@.service', state='enabled'),
        SystemdServiceFile(name='template@.service', state='enabled'),
    ]

    expected = [
        # dont expect example.service since it's static
        # dont expect abc.service since it's transient
        SystemdServicePreset(service='template@.service', state='disable'),
        SystemdServicePreset(service='template@instance1.service', state='enable'),
        SystemdServicePreset(service='template@instance2.service', state='enable'),
        SystemdServicePreset(service='globbed-one.service', state='enable'),
        SystemdServicePreset(service='globbed-two.service', state='enable'),
        SystemdServicePreset(service='extra.service', state='disable'),
        SystemdServicePreset(service='template2@.service', state='disable')
    ]

    presets = systemd.get_system_service_preset_files(service_files, False)
    assert sorted(presets, key=lambda e: e.service) == sorted(expected, key=lambda e: e.service)


def test_get_service_preset_files_invalid(monkeypatch):

    def get_system_preset_files_mocked():
        return [TESTING_PRESET_WITH_INVALID_ENTRIES]

    monkeypatch.setattr(systemd, '_get_system_preset_files', get_system_preset_files_mocked)
    monkeypatch.setattr(systemd, '_parse_preset_files', parse_preset_files_mocked())

    with pytest.raises(ValueError):
        # doesn't matter what service_files are
        systemd.get_system_service_preset_files([], ignore_invalid_entries=False)



from leapp.models import InstalledRPM, RPM
from leapp.reporting import Report
from leapp.snactor.fixture import current_actor_context


def test_actor_with_docker_package(current_actor_context):
    with_docker = [
            RPM(name='docker',
                epoch='2',
                packager='Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>',
                version='1.13.1',
                release='209.git7d71120.el7_9',
                arch='x86_64',
                pgpsig='RSA/SHA256, Fri 07 Jan 2022 01:50:17 PM UTC, Key ID 199e2f91fd431d51',
                repository='installed',
                module=None,
                stream=None),
            RPM(name='grep',
                epoch='0',
                packager='Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>',
                version='2.20',
                release='3.el7',
                arch='x86_64',
                pgpsig='RSA/SHA256, Fri 24 Mar 2017 04:59:11 PM UTC, Key ID 199e2f91fd431d51',
                repository='anaconda/7.9',
                module=None,
                stream=None)
            ]

    current_actor_context.feed(InstalledRPM(items=with_docker))
    current_actor_context.run()
    assert current_actor_context.consume(Report)


def test_actor_without_docker_package(current_actor_context):
    without_docker = [
            RPM(name='tree',
                epoch='0',
                packager='Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>',
                version='1.6.0',
                release='10.el7',
                arch='x86_64',
                pgpsig='RSA/SHA256, Wed 02 Apr 2014 09:33:48 PM UTC, Key ID 199e2f91fd431d51',
                repository='installed',
                module=None,
                stream=None),
            RPM(name='grep',
                epoch='0',
                packager='Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>',
                version='2.20',
                release='3.el7',
                arch='x86_64',
                pgpsig='RSA/SHA256, Fri 24 Mar 2017 04:59:11 PM UTC, Key ID 199e2f91fd431d51',
                repository='anaconda/7.9',
                module=None,
                stream=None)
            ]

    current_actor_context.feed(InstalledRPM(items=without_docker))
    current_actor_context.run()
    assert not current_actor_context.consume(Report)



from leapp.actors import Actor
from leapp.libraries.actor import scanrhsm
from leapp.models import RHSMInfo
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class ScanSubscriptionManagerInfo(Actor):
    """
    Scans the current system for subscription manager information

    Retrieves information about enabled and available repositories, attached SKUs, product certificates and release
    from the current system without modifying it.
    """

    name = 'scan_subscription_manager_info'
    consumes = ()
    produces = (RHSMInfo,)
    tags = (IPUWorkflowTag, FactsPhaseTag)

    def process(self):
        scanrhsm.scan()



from leapp import reporting
from leapp.libraries.actor import checkmemory
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked
from leapp.libraries.stdlib import api
from leapp.models import MemoryInfo
from leapp.utils.report import is_inhibitor


def test_check_memory_low(monkeypatch):
    minimum_req_error = []
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked())
    minimum_req_error = checkmemory._check_memory(MemoryInfo(mem_total=1024))
    assert minimum_req_error


def test_check_memory_high(monkeypatch):
    minimum_req_error = []
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked())
    minimum_req_error = checkmemory._check_memory(MemoryInfo(mem_total=16273492))
    assert not minimum_req_error


def test_report(monkeypatch):
    title_msg = 'Minimum memory requirements for RHEL 8 are not met'
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked())
    monkeypatch.setattr(api, 'consume', lambda x: iter([MemoryInfo(mem_total=129)]))
    monkeypatch.setattr(reporting, "create_report", create_report_mocked())
    checkmemory.process()
    assert reporting.create_report.called
    assert title_msg == reporting.create_report.report_fields['title']
    assert is_inhibitor(reporting.create_report.report_fields)



from leapp.models import fields, Model, RPM
from leapp.topics import TransactionTopic


class LeftoverPackages(Model):
    topic = TransactionTopic
    items = fields.List(fields.Model(RPM), default=[])


class RemovedPackages(LeftoverPackages):
    pass



from leapp.actors import Actor
from leapp.libraries.actor import scanpkgmanager
from leapp.models import PkgManagerInfo
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class ScanPkgManager(Actor):
    """
    Provides data about package manager (yum/dnf)
    """

    name = 'scan_pkg_manager'
    consumes = ()
    produces = (PkgManagerInfo,)
    tags = (IPUWorkflowTag, FactsPhaseTag)

    def process(self):
        scanpkgmanager.process()



import pytest

from leapp import reporting
from leapp.libraries.actor import checktargetrepos
from leapp.libraries.common import rhsm
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked
from leapp.libraries.stdlib import api
from leapp.models import (
    CustomTargetRepository,
    CustomTargetRepositoryFile,
    EnvVar,
    Report,
    RepositoryData,
    RHELTargetRepository,
    TargetRepositories
)
from leapp.utils.report import is_inhibitor


class MockedConsume(object):
    def __init__(self, *args):
        self._msgs = []
        for arg in args:
            if not arg:
                continue
            if isinstance(arg, list):
                self._msgs.extend(arg)
            else:
                self._msgs.append(arg)

    def __call__(self, model):
        return iter([msg for msg in self._msgs if isinstance(msg, model)])


_RHEL_REPOS = [
    RHELTargetRepository(repoid='repo1'),
    RHELTargetRepository(repoid='repo2'),
    RHELTargetRepository(repoid='repo3'),
    RHELTargetRepository(repoid='repo4'),
]

_CUSTOM_REPOS = [
    CustomTargetRepository(repoid='repo1', name='repo1name', baseurl='repo1url', enabled=True),
    CustomTargetRepository(repoid='repo2', name='repo2name', baseurl='repo2url', enabled=False),
    CustomTargetRepository(repoid='repo3', name='repo3name', baseurl=None, enabled=True),
    CustomTargetRepository(repoid='repo4', name='repo4name', baseurl=None, enabled=True),
]

_TARGET_REPOS_CUSTOM = TargetRepositories(rhel_repos=_RHEL_REPOS, custom_repos=_CUSTOM_REPOS)
_TARGET_REPOS_NO_CUSTOM = TargetRepositories(rhel_repos=_RHEL_REPOS)
_CUSTOM_TARGET_REPOFILE = CustomTargetRepositoryFile(file='/etc/leapp/files/leapp_upgrade_repositories.repo')


def test_checktargetrepos_rhsm(monkeypatch):
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr(rhsm, 'skip_rhsm', lambda: False)
    monkeypatch.setattr(api, 'consume', MockedConsume())
    monkeypatch.setattr(checktargetrepos, 'get_target_major_version', lambda: '8')
    checktargetrepos.process()
    assert reporting.create_report.called == 0


@pytest.mark.parametrize('enable_repos', [True, False])
@pytest.mark.parametrize('custom_target_repos', [True, False])
@pytest.mark.parametrize('custom_target_repofile', [True, False])
def test_checktargetrepos_no_rhsm(monkeypatch, enable_repos, custom_target_repos, custom_target_repofile):
    mocked_consume = MockedConsume(_TARGET_REPOS_CUSTOM if custom_target_repos else _TARGET_REPOS_NO_CUSTOM)
    if custom_target_repofile:
        mocked_consume._msgs.append(_CUSTOM_TARGET_REPOFILE)
    envars = {'LEAPP_ENABLE_REPOS': 'hill,spencer'} if enable_repos else {}
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(envars=envars))

    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr(rhsm, 'skip_rhsm', lambda: True)
    monkeypatch.setattr(api, 'consume', mocked_consume)
    monkeypatch.setattr(checktargetrepos, 'get_target_major_version', lambda: '8')

    checktargetrepos.process()

    if not custom_target_repos:
        assert reporting.create_report.called == 1
        assert is_inhibitor(reporting.create_report.report_fields)
    elif not enable_repos and custom_target_repos and not custom_target_repofile:
        assert reporting.create_report.called == 1
        assert not is_inhibitor(reporting.create_report.report_fields)
    else:
        assert reporting.create_report.called == 0



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class IfCfgProperty(Model):
    """
    Key-value pair for ifcfg properties.

    This model is not expected to be used as a message (produced/consumed by actors).
    It is used from within the IfCfg model.
    """
    topic = SystemInfoTopic

    name = fields.String()
    """ Name of a property """
    value = fields.Nullable(fields.String())
    """ Value of a property """


class IfCfg(Model):
    """
    IfCfg file describing legacy network configuration

    Produced for every ifcfg file loaded from key-value ("sysconfig")
    format described in nm-settings-ifcfg-rh(5) manual.
    """
    topic = SystemInfoTopic

    filename = fields.String()
    """ Path to file this model was populated from """
    properties = fields.List(fields.Model(IfCfgProperty), default=[])
    """ The list of name-value pairs from ifcfg file """
    secrets = fields.Nullable(fields.List(fields.Model(IfCfgProperty)))
    """ The list of name-value pairs from keys file """
    rules = fields.Nullable(fields.List(fields.String()))
    """ The list of traffic rules for IPv4 """
    rules6 = fields.Nullable(fields.List(fields.String()))
    """ The list of traffic rules for IPv6 """
    routes = fields.Nullable(fields.List(fields.String()))
    """ The list of routes for IPv4 """
    routes6 = fields.Nullable(fields.List(fields.String()))
    """ The list of routes for IPv6 """



from leapp.models import fields, Model
from leapp.topics import SystemFactsTopic


class InstalledKdeAppsFacts(Model):
    topic = SystemFactsTopic
    installed_apps = fields.List(fields.String(), default=[])



"""
Actor to check if KDE and/or GNOME are installed
Author: Jan Beran
Email: jaberan@redhat.com
"""

from leapp.actors import Actor
from leapp.libraries.actor.checkkdegnome import check_kde_gnome
from leapp.models import InstalledDesktopsFacts, InstalledKdeAppsFacts
from leapp.reporting import Report
from leapp.tags import ChecksPhaseTag, IPUWorkflowTag


class CheckKdeGnome(Actor):
    """
    Checks whether KDE is installed

    Actor will check whether KDE is installed together with GNOME desktop to inform whether we can
    inhibit the upgrade process. When both are installed, we need to inform the user that KDE will
    be removed and GNOME will be used instead. If only KDE is installed, we want to inhibit
    the upgrade process otherwise the user will end up without a desktop.
    Note: The Package Evolution Service data makes sure the KDE-related packages are removed in the
    dnf upgrade transaction.
    """
    name = 'check_kde_gnome'
    consumes = (InstalledDesktopsFacts, InstalledKdeAppsFacts)
    produces = (Report,)
    tags = (IPUWorkflowTag, ChecksPhaseTag)

    def process(self):
        check_kde_gnome()



from collections import namedtuple

import pytest

from leapp import reporting
from leapp.libraries.actor import checkinstalledkernels
from leapp.libraries.common.config import architecture
from leapp.libraries.common.testutils import create_report_mocked, CurrentActorMocked, logger_mocked
from leapp.libraries.stdlib import api
from leapp.models import DistributionSignedRPM, KernelInfo, RPM

RH_PACKAGER = 'Red Hat, Inc. <http://bugzilla.redhat.com/bugzilla>'

# Does not make sense to run any tests when the module is not accessible
pytest.importorskip("rpm")


# Partial RPM description, missing fields are filled with defaults
RPMDesc = namedtuple('RPMDesc', ('name', 'version', 'release', 'arch'))


def create_rpm(rpm_desc, packager=RH_PACKAGER, pgpsig='SOME_OTHER_SIG_X', epoch='0'):
    return RPM(name=rpm_desc.name, arch=rpm_desc.arch, version=rpm_desc.version, release=rpm_desc.release,
               epoch='0', packager=RH_PACKAGER, pgpsig='SOME_OTHER_SIG_X')


def create_rpms(rpm_descriptions):
    rpms = [create_rpm(rpm_desc) for rpm_desc in rpm_descriptions]
    installed_rpms = DistributionSignedRPM(items=rpms)
    return installed_rpms


s390x_pkgs_single = [
    RPMDesc(name='kernel', version='3.10.0', release='957.43.1.el7', arch='s390x'),
    RPMDesc(name='something', version='3.10.0', release='957.43.1.el7', arch='s390x'),
    RPMDesc(name='kernel-something', version='3.10.0', release='957.43.1.el7', arch='s390x'),
]
s390x_pkgs_multi = [
    RPMDesc(name='kernel', version='3.10.0', release='957.43.1.el7', arch='s390x'),
    RPMDesc(name='something', version='3.10.0', release='957.43.1.el7', arch='s390x'),
    RPMDesc(name='kernel', version='3.10.0', release='956.43.1.el7', arch='s390x')
]


@pytest.mark.parametrize(
    ('pkgs', 'should_inhibit'),  # First tuple in pkgs is expected to provide the booted kernel
    (
        (s390x_pkgs_single, False),
        (s390x_pkgs_multi, True)
    )
)
def test_s390x_kernel_count_inhibition(monkeypatch, pkgs, should_inhibit):
    installed_rpms_msg = create_rpms(pkgs)
    kernel_pkg = installed_rpms_msg.items[0]
    kernel_info = KernelInfo(pkg=kernel_pkg, uname_r='957.43.1.el7.s390x')

    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(arch=architecture.ARCH_S390X,
                                                                 msgs=[kernel_info, installed_rpms_msg]))
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    checkinstalledkernels.process()
    assert should_inhibit == bool(reporting.create_report.called)


versioned_kernel_pkgs = [
    RPMDesc(name='kernel', version='3.10.0', release='789.35.2.el7', arch='x86_64'),
    RPMDesc(name='kernel', version='3.10.0', release='1234.21.1.el7', arch='x86_64'),
    RPMDesc(name='kernel', version='4.14.0', release='115.29.1.el7', arch='x86_64'),  # [2] - newest
    RPMDesc(name='kernel', version='3.10.0', release='456.43.1.el7', arch='x86_64'),
]


@pytest.mark.parametrize(
    ('expect_report', 'installed_rpms_msg', 'current_kernel_pkg_index'),
    (
        (False, create_rpms(versioned_kernel_pkgs), 2),
        (True, create_rpms(versioned_kernel_pkgs), 1),
        (True, create_rpms(versioned_kernel_pkgs), 0),
    )
)
def test_newest_kernel(monkeypatch, expect_report, installed_rpms_msg, current_kernel_pkg_index):
    uname_r = ''  # Kernel release is not used to determine the kernel novelty
    kernel_info = KernelInfo(pkg=installed_rpms_msg.items[current_kernel_pkg_index], uname_r=uname_r)
    msgs = [installed_rpms_msg, kernel_info]
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=msgs))
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    checkinstalledkernels.process()
    if expect_report:
        assert reporting.create_report.called
        assert reporting.create_report.report_fields['title'] == 'Newest installed kernel not in use'
    else:
        assert not reporting.create_report.called


@pytest.mark.parametrize(
    'rpm_desc',
    [
        RPMDesc(name='', version='3.10.0', release='1234.21.1.el7', arch='x86_64'),
        RPMDesc(name='', version='3.10.0', release='456.43.1.el7', arch='x86_64'),
        RPMDesc(name='', version='3.10.0', release='1.1.1.1.1.1.1.2', arch='x86_64'),
        RPMDesc(name='', version='4.10.4', release='1234.21.1.el7', arch='x86_64'),
        RPMDesc(name='', version='6.6.6', release='1234.56.rt78.el9', arch='x86_64'),
    ]
)
def test_get_evr(monkeypatch, rpm_desc):
    pkg = create_rpm(rpm_desc)
    assert checkinstalledkernels.get_EVR(pkg) == ('', pkg.version, pkg.release)


versioned_kernel_rt_pkgs = [
    RPMDesc(name='kernel-rt', version='3.10.0', release='789.35.2.rt56.1133.el7', arch='x86_64'),
    RPMDesc(name='kernel-rt', version='3.10.0', release='789.35.2.rt57.1133.el7', arch='x86_64'),
    RPMDesc(name='kernel-rt', version='3.10.0', release='789.35.2.rt101.1133.el7', arch='x86_64'),
    RPMDesc(name='kernel-rt', version='3.10.0', release='790.35.2.rt666.1133.el7', arch='x86_64'),  # [3] - newest
]


@pytest.mark.parametrize(
    ('msgs', 'num', 'name'),
    [
        ([create_rpms(versioned_kernel_rt_pkgs)], 4, 'kernel-rt'),
        ([create_rpms(versioned_kernel_rt_pkgs[0:-1])], 3, 'kernel-rt'),
        ([create_rpms(versioned_kernel_rt_pkgs[0:-2])], 2, 'kernel-rt'),
        ([create_rpms(versioned_kernel_rt_pkgs[0:-3])], 1, 'kernel-rt'),
        ([create_rpms(versioned_kernel_rt_pkgs)], 0, 'kernel'),
        ([create_rpms(versioned_kernel_rt_pkgs)], 0, 'smth'),
        ([create_rpms(versioned_kernel_pkgs)], 0, 'kernel-rt'),
        ([create_rpms(versioned_kernel_pkgs + versioned_kernel_rt_pkgs)], 4, 'kernel-rt'),
        ([create_rpms(versioned_kernel_pkgs + versioned_kernel_rt_pkgs)], 4, 'kernel'),
    ]
)
def test_get_pkgs(monkeypatch, msgs, num, name):
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=msgs))
    pkgs = checkinstalledkernels.get_all_pkgs_with_name(name)
    assert len(pkgs) == num


mixed_kernel_pkgs = create_rpms(versioned_kernel_rt_pkgs + versioned_kernel_pkgs)
mixed_kernel_pkgs_desc_table = {  # Maps important pkgs from mixed_kernel_pkgs to their index so they can be ref'd
    'newest_rt': 3,
    'older_rt': 2,
    'newest_ordinary': 6,
    'older_ordinary': 5,
}


@pytest.mark.parametrize(
    ('expect_report', 'installed_rpms_msg', 'curr_kernel_pkg_index'),
    [
        # kernel-rt only
        (True, create_rpms(versioned_kernel_rt_pkgs), 0),
        (True, create_rpms(versioned_kernel_rt_pkgs), 1),
        (True, create_rpms(versioned_kernel_rt_pkgs), 2),
        (False, create_rpms(versioned_kernel_rt_pkgs), 3),  # newest
        (False, create_rpms(versioned_kernel_rt_pkgs[0:-1]), 2),
        (False, create_rpms(versioned_kernel_rt_pkgs[0:1]), 0),

        # mix of kernel-rt + kernel
        (True, mixed_kernel_pkgs, mixed_kernel_pkgs_desc_table['older_rt']),
        (False, mixed_kernel_pkgs, mixed_kernel_pkgs_desc_table['newest_rt']),
        (True, mixed_kernel_pkgs, mixed_kernel_pkgs_desc_table['older_ordinary']),
        (False, mixed_kernel_pkgs, mixed_kernel_pkgs_desc_table['newest_ordinary']),
    ]
)
def test_newest_kernel_realtime(monkeypatch, expect_report, installed_rpms_msg, curr_kernel_pkg_index):
    current_kernel_pkg = installed_rpms_msg.items[curr_kernel_pkg_index]
    kernel_info = KernelInfo(pkg=current_kernel_pkg, uname_r='')
    msgs = [installed_rpms_msg, kernel_info]

    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=msgs))
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    checkinstalledkernels.process()

    if expect_report:
        assert reporting.create_report.called
        assert reporting.create_report.report_fields['title'] == 'Newest installed kernel not in use'
    else:
        assert not reporting.create_report.called



from leapp.models import ActiveKernelModule, ActiveKernelModulesFacts
from leapp.reporting import Report
from leapp.snactor.fixture import current_actor_context
from leapp.utils.report import is_inhibitor


def create_modulesfacts(kernel_modules):
    return ActiveKernelModulesFacts(kernel_modules=kernel_modules)


def test_actor_with_btrfs_module(current_actor_context):
    with_btrfs = [
        ActiveKernelModule(filename='btrfs', parameters=[]),
        ActiveKernelModule(filename='kvm', parameters=[])]

    current_actor_context.feed(create_modulesfacts(kernel_modules=with_btrfs))
    current_actor_context.run()
    report_fields = current_actor_context.consume(Report)[0].report
    assert is_inhibitor(report_fields)


def test_actor_without_btrfs_module(current_actor_context):
    without_btrfs = [
        ActiveKernelModule(filename='kvm_intel', parameters=[]),
        ActiveKernelModule(filename='kvm', parameters=[])]

    current_actor_context.feed(create_modulesfacts(kernel_modules=without_btrfs))
    current_actor_context.run()
    assert not current_actor_context.consume(Report)



from leapp.actors import Actor
from leapp.exceptions import StopActorExecutionError
from leapp.models import FIPSInfo, KernelCmdline
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class ScanFIPS(Actor):
    """
    Determine whether the source system has FIPS enabled.
    """

    name = 'scan_fips'
    consumes = (KernelCmdline,)
    produces = (FIPSInfo,)
    tags = (IPUWorkflowTag, FactsPhaseTag)

    def process(self):
        cmdline = next(self.consume(KernelCmdline), None)
        if not cmdline:
            raise StopActorExecutionError('Cannot check FIPS state due to missing command line parameters',
                                          details={'Problem': 'Did not receive a message with kernel command '
                                                              'line parameters (KernelCmdline)'})

        for parameter in cmdline.parameters:
            if parameter.key == 'fips' and parameter.value == '1':
                self.produce(FIPSInfo(is_enabled=True))
                return
        self.produce(FIPSInfo(is_enabled=False))



from leapp.actors import Actor
from leapp.libraries.actor.xfsinfoscanner import scan_xfs
from leapp.models import StorageInfo, XFSPresence
from leapp.tags import FactsPhaseTag, IPUWorkflowTag


class XFSInfoScanner(Actor):
    """
    This actor scans all mounted mountpoints for XFS information

    The actor will check each mountpoint reported in the StorageInfo message, if the mountpoint is a partition with XFS
    using ftype = 0. The actor will produce a message with the findings.
    It will contain a list of all XFS mountpoints with ftype = 0 so that those mountpoints can be handled appropriately
    for the overlayfs that is going to be created.
    """

    name = 'xfs_info_scanner'
    consumes = (StorageInfo,)
    produces = (XFSPresence,)
    tags = (FactsPhaseTag, IPUWorkflowTag,)

    def process(self):
        scan_xfs()



import pytest

from leapp.libraries.common.config import get_all_envs, get_env, get_product_type
from leapp.libraries.common.testutils import CurrentActorMocked
from leapp.libraries.stdlib import api


def test_env_var_match(monkeypatch):
    envars = {'LEAPP_DEVEL_SKIP_WIP': '0',
              'LEAPP_DEVEL_SKIP_DIP': '1',
              'LEAPP_DEVEL_SKIP_RIP': '2'}
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(envars=envars))
    assert get_env('LEAPP_DEVEL_SKIP_WIP') == '0'
    assert not get_env('LEAPP_DEVEL_SKIP_PIP')


def test_get_all_vars(monkeypatch):
    envars = {'LEAPP_DEVEL_SKIP_WIP': '0',
              'LEAPP_DEVEL_SKIP_DIP': '1',
              'LEAPP_DEVEL_SKIP_RIP': '2'}
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(envars=envars))
    assert api.current_actor().configuration.leapp_env_vars == get_all_envs()
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked)
    assert api.current_actor().configuration.leapp_env_vars == get_all_envs()


def test_get_product_type_valid(monkeypatch):
    prod_types = ('ga', 'beta', 'htb', 'GA', 'BETA', 'HTB', '')
    for src, dst in [(i, j) for i in prod_types for j in prod_types]:
        envars = {'LEAPP_DEVEL_SOURCE_PRODUCT_TYPE': src,
                  'LEAPP_DEVEL_TARGET_PRODUCT_TYPE': dst}
        exp_src = 'ga' if not src else src.lower()
        exp_dst = 'ga' if not dst else dst.lower()
        monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(envars=envars))
        assert exp_src == get_product_type('source')
        assert exp_dst == get_product_type('target')
    # return 'ga' if envars are not specified
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked)
    assert get_product_type('source') == 'ga'
    assert get_product_type('target') == 'ga'


def test_get_product_type_invalid_product(monkeypatch):
    for sys_type, envar in [('source', 'LEAPP_DEVEL_SOURCE_PRODUCT_TYPE'),
                            ('target', 'LEAPP_DEVEL_TARGET_PRODUCT_TYPE')]:
        monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(envars={envar: 'wrong'}))
        with pytest.raises(ValueError) as err:
            get_product_type(sys_type)
        assert 'Invalid value in the {} envar'.format(envar) in str(err)


def test_get_product_type_invalid_param():
    with pytest.raises(ValueError) as err:
        get_product_type('fail')
    assert 'Given invalid sys_type.' in str(err)



import base64
import io
import os
import re
import tarfile
import tempfile

from leapp import reporting
from leapp.libraries.actor import checkntp
from leapp.libraries.common.testutils import create_report_mocked


def test_nomigration(monkeypatch):
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
    monkeypatch.setattr(checkntp, 'check_service', lambda _: False)
    monkeypatch.setattr(checkntp, 'is_file', lambda _: False)
    monkeypatch.setattr(checkntp, 'get_tgz64', lambda _: '')

    checkntp.check_ntp(set(['chrony', 'linuxptp', 'xterm']))

    assert reporting.create_report.called == 0


def test_migration(monkeypatch):
    for packages, services, migrate in [
                (['ntp'], ['ntpd'], ['ntpd']),
                (['ntp', 'ntpdate'], ['ntpd'], ['ntpd']),
                (['ntpdate'], ['ntpdate'], ['ntpdate']),
                (['ntp', 'ntpdate'], ['ntpdate'], ['ntpdate']),
                (['ntp', 'ntpdate'], ['ntpd', 'ntpdate'], ['ntpd', 'ntpdate']),
                (['ntp', 'ntpdate', 'ntp-perl'], ['ntpd', 'ntpdate'], ['ntpd', 'ntpdate']),
                (['ntp', 'ntpdate'], ['ntpd', 'ntpdate', 'ntp-wait'], ['ntpd', 'ntpdate']),
                (['ntp', 'ntpdate', 'ntp-perl'], ['ntpd', 'ntpdate', 'ntp-wait'], ['ntpd', 'ntpdate', 'ntp-wait']),
            ]:
        monkeypatch.setattr(reporting, 'create_report', create_report_mocked())
        monkeypatch.setattr(checkntp, 'check_service', lambda service: service[:-8] in services)
        monkeypatch.setattr(checkntp, 'is_file', lambda _: True)
        monkeypatch.setattr(checkntp, 'get_tgz64', lambda _: '')

        decision = checkntp.check_ntp(set(packages))

        assert reporting.create_report.called == 1
        assert 'configuration will be migrated' in reporting.create_report.report_fields['title']
        for service in ['ntpd', 'ntpdate']:
            migrated = re.search(r'\b{}\b'.format(service),
                                 reporting.create_report.report_fields['title']) is not None
            assert migrated == (service in migrate)

        assert decision.migrate_services == migrate


def test_tgz64(monkeypatch):
    f, name = tempfile.mkstemp()
    os.close(f)
    tgz64 = checkntp.get_tgz64([name])

    stream = io.BytesIO(base64.b64decode(tgz64))
    tar = tarfile.open(fileobj=stream, mode='r:gz')
    names = tar.getnames()

    tar.close()
    os.unlink(name)

    assert names == [name.lstrip('/')]



import os
import re
import textwrap

from six import StringIO

from leapp.libraries.common import utils
from leapp.libraries.stdlib import CalledProcessError, run
from leapp.models import Authselect


def read_file(config):
    """
        Read file contents. Return empty string if the file does not exist.
    """
    if not os.path.isfile(config):
        return ""
    with open(config) as f:
        return f.read()


def is_service_enabled(service):
    """
        Return true if @service is enabled with systemd, false otherwise.
    """
    try:
        run(["/usr/bin/systemctl", "is-enabled", "{}.service".format(service)])
    except (OSError, CalledProcessError):
        return False

    return True


class ConfigFile(object):
    """
        Base class for config parsers.
    """

    def __init__(self, content):
        parser = utils.parse_config(StringIO(textwrap.dedent(content)))
        self.config = parser

    def get_string(self, section, option):
        if not self.config.has_option(section, option):
            return None

        return self.config.get(section, option).strip('"\'')

    def get_bool(self, section, option):
        if not self.config.has_option(section, option):
            return False

        return self.config.getboolean(section, option)


class Authconfig(ConfigFile):
    """
        Parse authconfig configuration.
    """

    def __init__(self, config):
        # We add a custom section to convert the config to ini format
        super(Authconfig, self).__init__('[authconfig]\n' + config)

    def get_string(self, option):
        return super(Authconfig, self).get_string('authconfig', option)

    def get_bool(self, option):
        return super(Authconfig, self).get_bool('authconfig', option)


class DConf(ConfigFile):
    """
        Parse dconf configuration.
    """


class AuthselectScannerLibrary(object):
    """
    Detect what authselect configuration should be suggested to administrator.

    1. Detect possible authselect profile by looking up modules in PAM
       or by checking that daemon is enabled.
       - pam_sss -> sssd
       - pam_winbind -> winbind
       - ypbind enabled -> nis

       If more then one module/daemon is detected that we will keep the
       configuration intact. No authselect profile can be applied.

    2. Detect authselect profile features by looking up modules in PAM
       or nsswitch.conf.
       - pam_faillock => with-faillock
       - pam_fprintd => with-fingerprint
       - pam_access => with-pamaccess
       - pam_mkhomedir => with-mkhomedir
       - pam_oddjob_mkhomedir => with-mkhomedir

    3. Check if there are any unknown PAM modules.
       If there are used PAM modules not used in authselect (such as pam_ldap),
       we must keep the configuration intact.

    4. Check if authconfig was used to create current configuration.
       If yes, we can automatically convert the configuration to authselect.
       If no, we need admin's confirmation.

       - Check that /etc/sysconfig/authconfig exists.
       - Check that PAM configuration uses authconfig files.
       - Check that PAM configuration was not touch after sysconfig file
         was created.
    """

    def __init__(self, known_modules, authconfig, dconf, pam, nsswitch):
        self.known_modules = known_modules
        self.ac = authconfig
        self.dconf = dconf
        self.pam = pam
        self.nsswitch = nsswitch

        self.profile = None
        self.features = []
        self.confirm = True

    def process(self):
        # Detect possible authselect configuration
        self.profile = self.step_detect_profile()
        self.features += self.step_detect_features()
        self.features += self.step_detect_sssd_features(self.profile)
        self.features += self.step_detect_winbind_features(self.profile)

        # Check if there is any module that is not known by authselect.
        # In this case we must left existing configuration intact.
        if self.pam.has_unknown_module(self.known_modules):
            self.profile = None
            self.features = []

        # Check if the proposed authselect configuration can be activated
        # automatically or admin's confirmation is required.
        self.confirm = self.step_detect_if_confirmation_is_required()

        # Remove duplicates
        self.features = sorted(set(self.features))

        return Authselect(
            profile=self.profile,
            features=self.features,
            confirm=self.confirm
        )

    def step_detect_profile(self):
        """
        Authselect supports three different profiles:
          - sssd
          - winbind
          - nis

        Only one of these profiles can be selected therefore if existing
        configuration contains combination of these daemons we can not
        suggest any profile and must keep existing configuration.
        """
        enabled_no = 0
        profile = None

        if self.pam.has('pam_sss'):
            profile = 'sssd'
            enabled_no += 1

        if self.pam.has('pam_winbind'):
            profile = 'winbind'
            enabled_no += 1

        if is_service_enabled('ypbind'):
            profile = 'nis'
            enabled_no += 1

        return profile if enabled_no == 1 else None

    def step_detect_features(self):
        pam_map = {
            'pam_faillock': 'with-faillock',
            'pam_fprintd': 'with-fingerprint',
            'pam_access': 'with-pamaccess',
            'pam_mkhomedir': 'with-mkhomedir',
            'pam_oddjob_mkhomedir': 'with-mkhomedir'
        }

        features = []

        for module, feature in pam_map.items():
            if self.pam.has(module):
                features.append(feature)

        return features

    def step_detect_sssd_features(self, profile):
        if profile != "sssd":
            return []

        # sudoers: sss
        result = re.search(
            "^[ \t]*sudoers[ \t]*:.*sss.*$",
            self.nsswitch,
            re.MULTILINE
        )

        features = []

        if result is not None:
            features.append("with-sudo")

        # SSSD Smartcard support
        # We enable smartcard support only if it was not handled by pam_pkcs11.
        # Otherwise pam_pkcs11 configuration must be converted manually.
        if not self.pam.has('pam_pkcs11'):
            if self.ac.get_bool('USESMARTCARD'):
                features.append("with-smartcard")

            if self.ac.get_bool('FORCESMARTCARD'):
                features.append("with-smartcard-required")

            if self.dconf.get_string(
                    'org/gnome/settings-daemon/peripherals/smartcard',
                    'removal-action'
            ) == 'lock-screen':
                features.append("with-smartcard-lock-on-removal")

        return features

    def step_detect_winbind_features(self, profile):
        if profile != "winbind":
            return []

        if self.ac.get_bool('WINBINDKRB5'):
            return ['with-krb5']

        return []

    def step_detect_if_confirmation_is_required(self):
        sysconfig = '/etc/sysconfig/authconfig'
        links = {
            '/etc/pam.d/fingerprint-auth': '/etc/pam.d/fingerprint-auth-ac',
            '/etc/pam.d/password-auth': '/etc/pam.d/password-auth-ac',
            '/etc/pam.d/postlogin': '/etc/pam.d/postlogin-ac',
            '/etc/pam.d/smartcard-auth': '/etc/pam.d/smartcard-auth-ac',
            '/etc/pam.d/system-auth': '/etc/pam.d/system-auth-ac'
        }

        # Check that authconfig was used to create the configuration
        if not os.path.isfile(sysconfig):
            return True

        # Check that all files are symbolic links to authconfig files
        for name, target in links.items():
            if not os.path.islink(name):
                return True

            if os.readlink(name) != target:
                return True

        # Check that all file were not modified after
        # /etc/sysconfig/authconfig was created.
        mtime = os.path.getmtime(sysconfig)
        for f in links.values():
            if os.path.getmtime(f) > mtime:
                return True

        return False



import os
from re import compile as regexp

from leapp.libraries.stdlib import run

_current_boot_matcher = regexp(r'BootCurrent: (?P<boot_current>([0-9A-F]*))')
_next_boot_matcher = regexp(r'BootNext: (?P<boot_next>([0-9A-F]*))')


def get_current_boot_match(string):
    match = _current_boot_matcher.match(string)
    if not match:
        return None
    captured_groups = match.groupdict()
    return captured_groups['boot_current']


def get_next_boot_match(string):
    match = _next_boot_matcher.match(string)
    if not match:
        return None
    captured_groups = match.groupdict()
    return captured_groups['boot_next']


def maybe_emit_updated_boot_entry():
    if not os.path.exists('/sbin/efibootmgr'):
        return

    efi_info = run(['/sbin/efibootmgr'], checked=False, split=True)
    if efi_info['exit_code'] != 0:
        # Not an efi system
        return

    current_boot, next_boot = None, None
    for line in efi_info['stdout']:
        current_match = get_current_boot_match(line)
        if current_match:
            current_boot = current_match

        next_match = get_next_boot_match(line)
        if next_match:
            next_boot = next_match

    if current_boot and not next_boot:
        # We set BootNext to CurrentBoot only if BootNext wasn't previously set
        run(['/sbin/efibootmgr', '-n', current_boot])



from leapp.models import (
    IfCfg,
    IfCfgProperty,
    NetworkManagerConnection,
    NetworkManagerConnectionProperty,
    NetworkManagerConnectionSetting
)
from leapp.reporting import Report
from leapp.utils.report import is_inhibitor


def test_no_conf(current_actor_context):
    """
    No report if there are no networks.
    """

    current_actor_context.run()
    assert not current_actor_context.consume(Report)


def test_no_wireless(current_actor_context):
    """
    No report if there's a keyfile, but it's not for a wireless connection.
    """

    not_wifi_nm_conn = NetworkManagerConnection(filename='/NM/wlan0.nmconn', settings=(
        NetworkManagerConnectionSetting(name='connection'),
    ))

    current_actor_context.feed(not_wifi_nm_conn)
    current_actor_context.run()
    assert not current_actor_context.consume(Report)


def test_keyfile_static_wep(current_actor_context):
    """
    Report if there's a static WEP keyfile.
    """

    static_wep_nm_conn = NetworkManagerConnection(filename='/NM/wlan0.nmconn', settings=(
        NetworkManagerConnectionSetting(name='wifi-security', properties=(
            NetworkManagerConnectionProperty(name='auth-alg', value='open'),
            NetworkManagerConnectionProperty(name='key-mgmt', value='none'),
            NetworkManagerConnectionProperty(name='wep-key-type', value='1'),
        )),
    ))

    current_actor_context.feed(static_wep_nm_conn)
    current_actor_context.run()
    report_fields = current_actor_context.consume(Report)[0].report
    assert is_inhibitor(report_fields)


def test_keyfile_dynamic_wep(current_actor_context):
    """
    Report if there's a dynamic WEP keyfile.
    """

    dynamic_wep_conn = NetworkManagerConnection(filename='/NM/wlan0.nmconn', settings=(
        NetworkManagerConnectionSetting(name='wifi-security', properties=(
            NetworkManagerConnectionProperty(name='key-mgmt', value='ieee8021x'),
        )),
    ))

    current_actor_context.feed(dynamic_wep_conn)
    current_actor_context.run()
    report_fields = current_actor_context.consume(Report)[0].report
    assert is_inhibitor(report_fields)


def test_ifcfg_static_wep_ask(current_actor_context):
    """
    Report if there's a static WEP sysconfig without stored key.
    """

    static_wep_ask_key_ifcfg = IfCfg(filename='/NM/ifcfg-wlan0', properties=(
        IfCfgProperty(name='TYPE', value='Wireless'),
        IfCfgProperty(name='ESSID', value='wep1'),
        IfCfgProperty(name='NAME', value='wep1'),
        IfCfgProperty(name='MODE', value='Managed'),
        IfCfgProperty(name='WEP_KEY_FLAGS', value='ask'),
        IfCfgProperty(name='SECURITYMODE', value='open'),
        IfCfgProperty(name='DEFAULTKEY', value='1'),
        IfCfgProperty(name='KEY_TYPE', value='key'),
    ))

    current_actor_context.feed(static_wep_ask_key_ifcfg)
    current_actor_context.run()
    report_fields = current_actor_context.consume(Report)[0].report
    assert is_inhibitor(report_fields)


def test_ifcfg_static_wep(current_actor_context):
    """
    Report if there's a static WEP sysconfig with a stored passphrase.
    """

    static_wep_ifcfg = IfCfg(filename='/NM/ifcfg-wlan0', secrets=(
        IfCfgProperty(name='KEY_PASSPHRASE1', value=None),
    ))

    current_actor_context.feed(static_wep_ifcfg)
    current_actor_context.run()
    report_fields = current_actor_context.consume(Report)[0].report
    assert is_inhibitor(report_fields)


def test_ifcfg_dynamic_wep(current_actor_context):
    """
    Report if there's a dynamic WEP sysconfig.
    """

    dynamic_wep_ifcfg = IfCfg(filename='/NM/ifcfg-wlan0', properties=(
        IfCfgProperty(name='ESSID', value='dynwep1'),
        IfCfgProperty(name='MODE', value='Managed'),
        IfCfgProperty(name='KEY_MGMT', value='IEEE8021X'),
        IfCfgProperty(name='TYPE', value='Wireless'),
        IfCfgProperty(name='NAME', value='dynwep1'),
    ))

    current_actor_context.feed(dynamic_wep_ifcfg)
    current_actor_context.run()
    report_fields = current_actor_context.consume(Report)[0].report
    assert is_inhibitor(report_fields)



import pytest

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.actor import checkfstabxfsoptions
from leapp.models import FstabEntry, StorageInfo
from leapp.reporting import Report
from leapp.snactor.fixture import current_actor_context
from leapp.utils.report import is_inhibitor


def _myint_gen():
    i = 0
    while True:
        yield i
        i += 1


def _gen_fs_ent(fstype='ext4', mntops='auto', val=_myint_gen()):
    return FstabEntry(
        fs_spec='/path/spec/{}'.format(next(val)),
        fs_file='/path/file/{}'.format(next(val)),
        fs_vfstype=fstype,
        fs_mntops=mntops,
        fs_freq='1',
        fs_passno='1',
    )


@pytest.mark.parametrize('fstab', [
    [_gen_fs_ent()],
    [_gen_fs_ent() for dummy in range(4)],
    [_gen_fs_ent(), _gen_fs_ent('ext4', 'auto,quota,huge_file')],
    # checking that problematic options are ignored for non-xfs FS
    [_gen_fs_ent(), _gen_fs_ent('ext4', 'auto,barier,huge_file')],
    [_gen_fs_ent('ext4', i) for i in checkfstabxfsoptions.REMOVED_XFS_OPTIONS],
    [_gen_fs_ent(i, 'nobarrier') for i in ('ext4', 'ext3', 'vfat', 'btrfs')],
])
def test_no_xfs_option(fstab, current_actor_context):
    current_actor_context.feed(StorageInfo(fstab=fstab))
    current_actor_context.run()
    report = current_actor_context.consume(Report)
    assert not report


# each item == one fstab
problematic_fstabs = [[_gen_fs_ent('xfs', ','.join(checkfstabxfsoptions.REMOVED_XFS_OPTIONS))]]
for opt in checkfstabxfsoptions.REMOVED_XFS_OPTIONS:
    problematic_fstabs.append([_gen_fs_ent('xfs', opt)])
    problematic_fstabs.append([_gen_fs_ent(), _gen_fs_ent('xfs', opt)])
    problematic_fstabs.append([_gen_fs_ent(), _gen_fs_ent('xfs', opt), _gen_fs_ent()])
    pre_opts = '{},auto,quota'.format(opt)
    in_opts = 'auto,{},quota'.format(opt)
    post_opts = 'auto,quota,{}'.format(opt)
    problematic_fstabs.append([_gen_fs_ent(), _gen_fs_ent('xfs', pre_opts)])
    problematic_fstabs.append([_gen_fs_ent(), _gen_fs_ent('xfs', in_opts)])
    problematic_fstabs.append([_gen_fs_ent(), _gen_fs_ent('xfs', post_opts)])
# ensure we catch even cases when a value is expected to be specified; we know just this
# one case, so it should be representative it's working like that..
problematic_fstabs.append([_gen_fs_ent(), _gen_fs_ent('xfs', 'defaults,ihashsize=4096')])
problematic_fstabs.append([_gen_fs_ent(), _gen_fs_ent('xfs', 'defaults,ihashsize=4096,auto')])


@pytest.mark.parametrize('fstab', problematic_fstabs)
def test_removed_xfs_option(fstab, current_actor_context):
    current_actor_context.feed(StorageInfo(fstab=fstab))
    current_actor_context.run()
    report = current_actor_context.consume(Report)
    assert report and len(report) == 1
    assert is_inhibitor(report[0].report)



from leapp.libraries.actor import updates
from leapp.libraries.common import isccfg
from leapp.models import BindFacts


def test_simple():
    """Test configuration is not modified without offending statements."""
    mockcfg = isccfg.MockConfig("""
options {
    listen-on port 53 { 127.0.0.1; };
    listen-on-v6 port 53 { ::1; };
    directory       "/var/named";
    allow-query     { localhost; };
    recursion yes;

    dnssec-validation yes;
};

zone "." IN {
    type hint;
    file "named.ca";
};
""", '/etc/named.conf')
    parser = isccfg.IscConfigParser(mockcfg)
    modified = updates.update_config(parser, mockcfg)
    assert modified == mockcfg.buffer


def test_dnssec_lookaside():
    """Test unsupported statements are removed."""
    mockcfg = isccfg.MockConfig("""
options {
    listen-on port 53 { 127.0.0.1; };
    listen-on-v6 port 53 { ::1; };
    directory       "/var/named";
    allow-query     { localhost; };
    recursion yes;

    dnssec-validation yes;
        dnssec-lookaside auto;
};

zone "." IN {
    type hint;
    file "named.ca";
};
""", '/etc/named.conf')
    parser = isccfg.IscConfigParser(mockcfg)
    modified = updates.update_config(parser, mockcfg)
    assert modified != mockcfg.buffer



import pytest

from leapp.libraries.actor.sanebackendsmigrate import (
    CANON,
    CANON_DR,
    CARDSCAN,
    DLL,
    EPJITSU,
    FUJITSU,
    update_config,
    XEROX_MFP
)


def _pattern_exists(content, macro):
    for line in content.split('\n'):
        if line.lstrip().startswith(macro):
            return True
    return False


def _create_original_file(file_content):
    content = ''
    for line in file_content:
        fmt_line = '{}\n'.format(line)
        content += fmt_line
    return content


def _create_expected_file(original_content, new_content):
    macros = []
    for line in new_content:
        if not _pattern_exists(original_content, line):
            macros.append(line)

    fmt_input = ''
    if macros:
        fmt_input = "\n{comment_line}\n{content}\n".format(comment_line='# content added by Leapp',
                                                           content='\n'.join(macros))

    return '\n'.join((original_content, fmt_input))


testdata = [
    (
        _create_original_file(['']),
        _create_expected_file('', CANON),
        CANON
    ),
    (
        _create_original_file(['']),
        _create_expected_file('', CANON_DR),
        CANON_DR
    ),
    (
        _create_original_file(['']),
        _create_expected_file('', CARDSCAN),
        CARDSCAN
    ),
    (
        _create_original_file(['']),
        _create_expected_file('', DLL),
        DLL
    ),
    (
        _create_original_file(['']),
        _create_expected_file('', EPJITSU),
        EPJITSU
    ),
    (
        _create_original_file(['']),
        _create_expected_file('', FUJITSU),
        FUJITSU
    ),
    (
        _create_original_file(['']),
        _create_expected_file('', XEROX_MFP),
        XEROX_MFP
    ),
    (
        _create_original_file(['fdfdfdr']),
        _create_expected_file('fdfdfdr', CANON),
        CANON
    ),
    (
        _create_original_file(['fdfdfdr']),
        _create_expected_file('fdfdfdr', CANON_DR),
        CANON_DR
    ),
    (
        _create_original_file(['fdfdfdr']),
        _create_expected_file('fdfdfdr', CARDSCAN),
        CARDSCAN
    ),
    (
        _create_original_file(['fdfdfdr']),
        _create_expected_file('fdfdfdr', DLL),
        DLL
    ),
    (
        _create_original_file(['fdfdfdr']),
        _create_expected_file('fdfdfdr', EPJITSU),
        EPJITSU
    ),
    (
        _create_original_file(['fdfdfdr']),
        _create_expected_file('fdfdfdr', FUJITSU),
        FUJITSU
    ),
    (
        _create_original_file(['fdfdfdr']),
        _create_expected_file('fdfdfdr', XEROX_MFP),
        XEROX_MFP
    ),
    (
        _create_original_file(['usb 0x04a9 0x2214']),
        _create_expected_file('usb 0x04a9 0x2214', CANON),
        CANON
    ),
    (
        _create_original_file(['usb 0x1083 0x162c']),
        _create_expected_file('usb 0x1083 0x162c', CANON_DR),
        CANON_DR
    ),
    (
        _create_original_file(['usb 0x0451 0x6250']),
        _create_expected_file('usb 0x0451 0x6250', CARDSCAN),
        CARDSCAN
    ),
    (
        _create_original_file(['#usb 0x0451 0x6250']),
        _create_expected_file('#usb 0x0451 0x6250', CARDSCAN),
        CARDSCAN
    ),
    (
        _create_original_file(['epsonds']),
        _create_expected_file('epsonds', DLL),
        DLL
    ),
    (
        _create_original_file(['usb 0x04c5 0x11bd']),
        _create_expected_file('usb 0x04c5 0x11bd', EPJITSU),
        EPJITSU
    ),
    (
        _create_original_file(['usb 0x04c5 0x132c']),
        _create_expected_file('usb 0x04c5 0x132c', FUJITSU),
        FUJITSU
    ),
    (
        _create_original_file(['usb 0x04e8 0x3471']),
        _create_expected_file('usb 0x04e8 0x3471', XEROX_MFP),
        XEROX_MFP
    ),
    (
        _create_original_file(CANON),
        _create_original_file(CANON),
        CANON
    ),
    (
        _create_original_file(CANON_DR),
        _create_original_file(CANON_DR),
        CANON_DR
    ),
    (
        _create_original_file(CARDSCAN),
        _create_original_file(CARDSCAN),
        CARDSCAN
    ),
    (
        _create_original_file(DLL),
        _create_original_file(DLL),
        DLL
    ),
    (
        _create_original_file(EPJITSU),
        _create_original_file(EPJITSU),
        EPJITSU
    ),
    (
        _create_original_file(FUJITSU),
        _create_original_file(FUJITSU),
        FUJITSU
    ),
    (
        _create_original_file(XEROX_MFP),
        _create_original_file(XEROX_MFP),
        XEROX_MFP
    )
]
"""
3-tuple of original file, file after migration and list of lines which
will be tried to add
"""


class MockFile(object):
    def __init__(self, path, content=None):
        self.path = path
        self.content = content
        self.error = False

    def append(self, path, content):
        if path != self.path:
            self.error = True
        if not self.error:
            self.content += content
            return self.content
        raise IOError('Error during writing to file: {}.'.format(path))

    def exists(self, path, macro):
        for line in self.content.split('\n'):
            if line.lstrip().startswith(macro) and self.path == path:
                return True
        return False


def test_update_config_file_errors():
    path = 'foo'
    new_content = ['fdfgdfg', 'gnbfgnf']

    f = MockFile(path, content='')

    with pytest.raises(IOError):
        update_config('bar', new_content, f.exists, f.append)

    assert f.content == ''


@pytest.mark.parametrize('orig_content,expected_result,content_to_add', testdata)
def test_update_config_append_into_file(orig_content,
                                        expected_result,
                                        content_to_add):
    f = MockFile('foo', orig_content)

    update_config('foo', content_to_add, f.exists, f.append)

    assert f.content == expected_result



from leapp.models import fields, Model
from leapp.topics import SystemInfoTopic


class FirewalldUsedObjectNames(Model):
    """
    This model contains lists of firewalld object (e.g. zones, services) names
    in use by the permanent firewalld configuration.
    """
    topic = SystemInfoTopic

    services = fields.List(fields.String(), default=[])
    """
    list of services (names) in use by firewalld's permanent configuration

    e.g. ["ssh", "https"]
    """

    policies = fields.List(fields.String(), default=[])
    """
    list of policies (names) in use by firewalld's permanent configuration

    e.g. ["allow-host-ipv6", "mypolicy"]
    """

    zones = fields.List(fields.String(), default=[])
    """
    list of zones (names) in use by firewalld's permanent configuration

    e.g. ["public", "internal", "nm-shared", "libvirt"]
    """



import mock
import six

from leapp.libraries.actor import scanmemory
from leapp.libraries.common import testutils
from leapp.libraries.stdlib import api
from leapp.models import MemoryInfo


def test_with_low_memory(monkeypatch):
    if six.PY3:
        with mock.patch("builtins.open", mock.mock_open(read_data="MemTotal: 42 kB")) as mock_proc_meminfo:
            monkeypatch.setattr(api, 'produce', testutils.produce_mocked())
            scanmemory.process()
            mock_proc_meminfo.assert_called_once_with('/proc/meminfo')
            assert api.produce.called == 1
            assert MemoryInfo(mem_total=42) == api.produce.model_instances[0]
    else:
        with mock.patch("__builtin__.open", mock.mock_open(read_data="MemTotal: 42 kB")) as mock_proc_meminfo:
            monkeypatch.setattr(api, 'produce', testutils.produce_mocked())
            scanmemory.process()
            mock_proc_meminfo.assert_called_once_with('/proc/meminfo')
            assert api.produce.called == 1
            assert MemoryInfo(mem_total=42) == api.produce.model_instances[0]


def test_with_high_memory(monkeypatch):
    if six.PY3:
        with mock.patch("builtins.open", mock.mock_open(read_data="MemTotal: 424242424242 kB")) as mock_proc_meminfo:
            monkeypatch.setattr(api, 'produce', testutils.produce_mocked())
            scanmemory.process()
            mock_proc_meminfo.assert_called_once_with('/proc/meminfo')
            assert api.produce.called == 1
            assert MemoryInfo(mem_total=424242424242) == api.produce.model_instances[0]
    else:
        with mock.patch(
            "__builtin__.open",
            mock.mock_open(read_data="MemTotal: 424242424242 kB"),
        ) as mock_proc_meminfo:
            monkeypatch.setattr(api, 'produce', testutils.produce_mocked())
            scanmemory.process()
            mock_proc_meminfo.assert_called_once_with('/proc/meminfo')
            assert api.produce.called == 1
            assert MemoryInfo(mem_total=424242424242) == api.produce.model_instances[0]



import os
import re
from shutil import rmtree

from leapp.libraries.common.config import version
from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import SELinuxModule

# types and attributes that where removed between RHEL 7 and 8
REMOVED_TYPES_EL7 = ["base_typeattr_15", "direct_run_init", "gpgdomain", "httpd_exec_scripts",
                     "httpd_user_script_exec_type", "ibendport_type", "ibpkey_type", "pcmcia_typeattr_2",
                     "pcmcia_typeattr_3", "pcmcia_typeattr_4", "pcmcia_typeattr_5", "pcmcia_typeattr_6",
                     "pcmcia_typeattr_7", "sandbox_caps_domain", "sandbox_typeattr_2", "sandbox_typeattr_3",
                     "sandbox_typeattr_4", "server_ptynode", "systemctl_domain", "user_home_content_type",
                     "userhelper_type", "cgdcbxd_exec_t", "cgdcbxd_t", "cgdcbxd_unit_file_t", "cgdcbxd_var_run_t",
                     "ganesha_use_fusefs", "ganesha_exec_t", "ganesha_t", "ganesha_tmp_t", "ganesha_unit_file_t",
                     "ganesha_var_log_t", "ganesha_var_run_t", "ganesha_use_fusefs"]

# types and attributes that where removed between RHEL 8 and 9
REMOVED_TYPES_EL8 = ["cephfs_t", "cgdcbxd_exec_t", "cgdcbxd_t", "cgdcbxd_unit_file_t", "cgdcbxd_var_run_t",
                     "cloud_what_var_cache_t", "journal_remote_client_packet_t", "journal_remote_port_t",
                     "journal_remote_server_packet_t", "kdbusfs_t", "logging_syslogd_list_non_security_dirs",
                     "nvme_device_t", "pcp_pmmgr_exec_t", "pcp_pmmgr_initrc_exec_t", "pcp_pmmgr_t",
                     "pcp_pmwebd_exec_t", "pcp_pmwebd_initrc_exec_t", "pcp_pmwebd_t", "rpm_transition_domain",
                     "systemd_journal_upload_exec_t", "systemd_journal_upload_t",
                     "systemd_journal_upload_var_lib_t", "virt_qmf_exec_t", "virt_qmf_t"]

# types, attributes and boolean contained in container-selinux
CONTAINER_TYPES = ["container_connect_any", "container_runtime_t", "container_runtime_exec_t", "spc_t",
                   "container_auth_t", "container_auth_exec_t", "spc_var_run_t", "container_var_lib_t",
                   "container_home_t", "container_config_t", "container_lock_t", "container_log_t",
                   "container_runtime_tmp_t", "container_runtime_tmpfs_t", "container_var_run_t",
                   "container_plugin_var_run_t", "container_unit_file_t", "container_devpts_t", "container_share_t",
                   "container_port_t", "container_build_t", "container_logreader_t", "docker_log_t", "docker_tmpfs_t",
                   "docker_share_t", "docker_t", "docker_lock_t", "docker_home_t", "docker_exec_t",
                   "docker_unit_file_t", "docker_devpts_t", "docker_config_t", "docker_tmp_t", "docker_auth_exec_t",
                   "docker_plugin_var_run_t", "docker_port_t", "docker_auth_t", "docker_var_run_t",
                   "docker_var_lib_t", "container_domain", "container_net_domain"]

WORKING_DIRECTORY = "/tmp/selinux/"

# list of policy modules used by udica
UDICA_TEMPLATES = {"base_container", "config_container", "home_container", "log_container",
                   "net_container", "tmp_container", "tty_container", "virt_container", "x_container"}


def check_module(name):
    """
    Check if given module contains one of removed types and comment out corresponding lines.

    The function expects a text file "$name" containing cil policy
    to be present in the current directory.

    Returns a list of invalid lines.
    """
    # get removed_types list based on upgrade path
    removed_types = REMOVED_TYPES_EL7 if version.get_source_major_version() == "7" else REMOVED_TYPES_EL8

    try:
        removed = run(["grep", "-w", "-E", "|".join(removed_types), name], split=True)
        # Add ";" at the beginning of invalid lines (comment them out)
        run(["sed", "-i", "/{}/s/^/;/g".format(r"\|".join(removed_types)), name])
        return removed.get("stdout", [])
    except CalledProcessError:
        return []


def list_selinux_modules():
    """
    Produce list of SELinux policy modules

    Returns list of tuples (name,priority)
    """
    try:
        semodule = run(['semodule', '-lfull'], split=True)
    except CalledProcessError as e:
        api.current_logger().warning('Cannot get list of selinux modules: {}'.format(e))
        return []

    modules = []
    for module in semodule.get("stdout", []):
        # Matching line such as "100 zebra             pp "
        # "<priority> <module name>    <module type - pp/cil> "
        m = re.match(r'([0-9]+)\s+([\w-]+)\s+([\w-]+)\s*\Z', module)
        if not m:
            # invalid output of "semodule -lfull"
            api.current_logger().warning('Invalid output of "semodule -lfull": {}'.format(module))
            continue
        modules.append((m.group(2), m.group(1)))

    return modules


def get_selinux_modules():
    """
    Read all custom SELinux policy modules from the system

    Returns a tuple (modules, install_rpms)
    where "modules" is a list of "SELinuxModule" objects
    and "install_rpms" is a list of RPMs
    that should be installed during the upgrade

    """

    modules = list_selinux_modules()
    # custom selinux policy modules
    semodule_list = []
    # udica templates
    template_list = []
    # list of rpms containing policy modules to be installed on RHEL 8
    install_rpms = []

    # modules need to be extracted into cil files
    # cd to /tmp/selinux and save working directory so that we can return there

    # clear working directory
    rmtree(WORKING_DIRECTORY, ignore_errors=True)

    try:
        wd = os.getcwd()
        os.mkdir(WORKING_DIRECTORY)
        os.chdir(WORKING_DIRECTORY)
    except OSError:
        api.current_logger().warning("Failed to access working directory! Aborting.")
        return ([], [], [])

    for (name, priority) in modules:
        # Udica templates should not be transferred, we only need a list of their
        # names and priorities so that we can reinstall their latest verisions
        if name in UDICA_TEMPLATES:
            template_list.append(
                SELinuxModule(
                    name=name,
                    priority=int(priority),
                    content='',
                    removed=[],
                )
            )
            continue

        if priority in ["100", "200"]:
            # 100 - module from selinux-policy-* package
            # 200 - DSP module - installed by an RPM - handled by PES
            continue
        # extract custom module and save it to SELinuxModule object
        module_file = name + ".cil"
        try:
            run(["semodule", "-c", "-X", priority, "-E", name])
            # check if the module contains invalid types and remove them if so
            removed = check_module(module_file)

            # get content of the module
            try:
                with open(module_file) as cil_file:
                    module_content = cil_file.read()
            except OSError as e:
                api.current_logger().warning("Error reading {}.cil : {}".format(name, e))
                continue

            semodule_list.append(
                SELinuxModule(
                    name=name,
                    priority=int(priority),
                    content=module_content,
                    removed=removed,
                )
            )
        except CalledProcessError:
            api.current_logger().warning("Module {} could not be extracted!".format(name))
            continue
        # rename the cil module file so that it does not clash
        # with the same module on different priority
        try:
            os.rename(module_file, "{}_{}".format(name, priority))
        except OSError:
            api.current_logger().warning(
                "Failed to rename module file {} to include priority.".format(name)
            )

    # Udica templates where moved to container-selinux package.
    # Make sure it is installed so that the templates can be reinstalled
    if template_list:
        install_rpms.append("container-selinux")

    # Process customizations introduced by "semanage"
    # this is necessary for check if container-selinux needs to be installed
    try:
        run(["semanage", "export", "-f", "semanage"])
    except CalledProcessError:
        pass
    # Check if modules contain any type, attribute, or boolean contained in container-selinux and install it if so
    # This is necessary since container policy module is part of selinux-policy-targeted in RHEL 7 (but not in RHEL 8)
    try:
        run(["grep", "-w", "-r", "-E", "|".join(CONTAINER_TYPES)], split=False)
        # Request "container-selinux" to be installed since container types where used in local customizations
        # and container-selinux policy was removed from selinux-policy-* packages
        install_rpms.append("container-selinux")
    except CalledProcessError:
        # expected, ignore exception
        pass

    try:
        os.chdir(wd)
    except OSError:
        pass
    rmtree(WORKING_DIRECTORY, ignore_errors=True)

    return (semodule_list, template_list, list(set(install_rpms)))


def get_selinux_customizations():
    """
    Extract local SELinux customizations introduced by semanage command

    Returns tuple (semanage_valid, semanage_removed)
    where "semanage_valid" is a list of semanage commands
    which should be safe to re-apply on RHEL 8 system
    and "semanage_removed" is a list of commands that
    will no longer be valid after system upgrade
    """
    removed_types = REMOVED_TYPES_EL7 if version.get_source_major_version() == "7" else REMOVED_TYPES_EL8

    semanage_removed = []
    semanage_valid = []
    try:
        # Collect SELinux customizations and select the ones that
        # can be reapplied after the upgrade
        semanage = run(["semanage", "export"], split=True)
        for line in semanage.get("stdout", []):
            # Skip "deleteall" commands to avoid removing customizations
            # done by package scripts during upgrade
            if " -D" in line:
                continue
            for setype in removed_types:
                if setype in line:
                    semanage_removed.append(line)
                    break
            else:
                semanage_valid.append(line)

    except CalledProcessError as e:
        api.current_logger().warning(
            "Failed to export SELinux customizations: {}".format(e.stderr)
        )

    return (semanage_valid, semanage_removed)



import errno

from leapp.libraries.actor import spamassassinconfigupdate_spamd
from leapp.libraries.common.spamassassinutils import SYSCONFIG_SPAMASSASSIN, SYSCONFIG_VARIABLE
from leapp.libraries.common.testutils import make_IOError, make_OSError
from leapp.models import SpamassassinFacts

# The tests for _drop_ssl_version and _drop_daemonize_option are overly
# restrictive in what output they accept - namely regarding whitespace and
# order of the options; don't be afraid to change the tests if they start
# failing due to whitespace or the order of options. It's done this way for
# simplicity - regular expressions accepting all acceptable outputs would be
# too complex.


def test_drop_ssl_version_double_quotes():
    value = '"--ssl --ssl-version tlsv1"'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == '"--ssl "'

    value = '"--ssl --ssl-version=tlsv1"'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == '"--ssl "'

    value = '"--ssl --ssl-version sslv3"'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == '"--ssl "'

    value = '"-d -c -m5 -H --ssl --ssl-version tlsv1"'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == '"-d -c -m5 -H --ssl "'

    value = '"-d -c -m5 -H --ssl --ssl-version sslv3"'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == '"-d -c -m5 -H --ssl "'

    value = '"-d --ssl -c -m5 --ssl-version tlsv1 -H"'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == '"-d --ssl -c -m5  -H"'

    value = '"-d --ssl -c -m5 --ssl-version sslv3 -H"'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == '"-d --ssl -c -m5  -H"'

    value = '"-d --ssl -c -m5 --ssl-version=sslv3 -H"'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == '"-d --ssl -c -m5  -H"'


def test_drop_ssl_version_no_quotes():
    value = '-d --ssl -c -m5 --ssl-version sslv3 -H'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == '-d --ssl -c -m5  -H'

    value = '-d --ssl -c -m5 --ssl-version=sslv3 -H'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == '-d --ssl -c -m5  -H'


def test_drop_ssl_version_single_quotes():
    value = "'-d --ssl -c -m5 --ssl-version sslv3 -H'"
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == "'-d --ssl -c -m5  -H'"

    value = "'-d --ssl -c -m5 --ssl-version=sslv3 -H'"
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == "'-d --ssl -c -m5  -H'"


def test_drop_ssl_version_ssl_version_implies_ssl():
    # If --ssl-version is used and --ssl is missing, we need to add --ssl
    # because --ssl-version implies --ssl
    value = '--ssl-version tlsv1'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == '--ssl'

    value = '--ssl-version=tlsv1'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == '--ssl'

    value = '"--ssl-version tlsv1"'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == '"--ssl"'

    value = ' --ssl-version tlsv1 '
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == ' --ssl '

    value = '-d -c -m5 --ssl-version sslv3 -H'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == '-d -c -m5 --ssl -H'

    value = '-d -c -m5 --ssl-version=sslv3 -H'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == '-d -c -m5 --ssl -H'

    value = '-d -c -m5 -H --ssl-version sslv3'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == '-d -c -m5 -H --ssl'


def test_drop_ssl_version_invalid_argument():
    # If the argument of --ssl-version is invalid, we don't touch it because
    # the configuration is invalid and the user needs to fix it up anyway.
    value = '--ssl-version foo'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == value

    value = '--ssl-version=foo'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == value

    value = '-d --ssl -c --ssl-version foo -H'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == value

    value = '-d --ssl -c --ssl-version=foo -H'
    rewritten = spamassassinconfigupdate_spamd._drop_ssl_version(value)
    assert rewritten == value


def test_drop_daemonize_option_double_quotes():
    value = '"-d -c -m5 -H"'
    rewritten = spamassassinconfigupdate_spamd._drop_daemonize_option(value)
    assert rewritten == '" -c -m5 -H"'

    value = '"-c -d -m5 -H"'
    rewritten = spamassassinconfigupdate_spamd._drop_daemonize_option(value)
    assert rewritten == '"-c  -m5 -H"'


def test_drop_daemonize_option_no_quotes():
    value = '-d -c -m5 -H'
    rewritten = spamassassinconfigupdate_spamd._drop_daemonize_option(value)
    assert rewritten == ' -c -m5 -H'

    value = '-c -d -m5 -H'
    rewritten = spamassassinconfigupdate_spamd._drop_daemonize_option(value)
    assert rewritten == '-c  -m5 -H'


def test_drop_daemonize_option_single_quotes():
    value = "'-d -c -m5 -H'"
    rewritten = spamassassinconfigupdate_spamd._drop_daemonize_option(value)
    assert rewritten == "' -c -m5 -H'"

    value = "'-c -d -m5 -H'"
    rewritten = spamassassinconfigupdate_spamd._drop_daemonize_option(value)
    assert rewritten == "'-c  -m5 -H'"


def test_drop_daemonize_option_longopt():
    value = '--daemonize -c -m5 -H'
    rewritten = spamassassinconfigupdate_spamd._drop_daemonize_option(value)
    assert rewritten == ' -c -m5 -H'

    value = '-c --daemonize -m5 -H'
    rewritten = spamassassinconfigupdate_spamd._drop_daemonize_option(value)
    assert rewritten == '-c  -m5 -H'


def test_drop_daemonize_short_form():
    value = '-cdL -m5'
    rewritten = spamassassinconfigupdate_spamd._drop_daemonize_option(value)
    assert rewritten == '-cL -m5'

    value = '-dcL -m5'
    rewritten = spamassassinconfigupdate_spamd._drop_daemonize_option(value)
    assert rewritten == '-cL -m5'

    value = '-cLd -m5'
    rewritten = spamassassinconfigupdate_spamd._drop_daemonize_option(value)
    assert rewritten == '-cL -m5'


def test_drop_daemonize_option_repeated():
    value = '--daemonize -cdd -m5 -dcd --daemonize -H -ddd'
    rewritten = spamassassinconfigupdate_spamd._drop_daemonize_option(value)
    assert rewritten == ' -c -m5 -c  -H '


def test_rewrite_spamd_option_no_assignment():
    content = '# foo\nbar=foobar\n'
    ops = [spamassassinconfigupdate_spamd._drop_ssl_version, spamassassinconfigupdate_spamd._drop_daemonize_option]
    rewritten = spamassassinconfigupdate_spamd._rewrite_spamd_option(content, ops)
    assert rewritten == content


def test_rewrite_spamd_option_comments():
    ops = [spamassassinconfigupdate_spamd._drop_ssl_version, spamassassinconfigupdate_spamd._drop_daemonize_option]

    # Leave comments be
    content = '# foo\n# bar\nSPAMDOPTIONS="--ssl-version tlsv1 -d --ssl"\n# foobar\n'
    rewritten = spamassassinconfigupdate_spamd._rewrite_spamd_option(content, ops)
    assert rewritten == '# foo\n# bar\nSPAMDOPTIONS="  --ssl"\n# foobar\n'

    content = '# foo\n# bar\n# SPAMDOPTIONS="--ssl-version tlsv1 -d --ssl"\n# foobar\n'
    rewritten = spamassassinconfigupdate_spamd._rewrite_spamd_option(content, ops)
    assert rewritten == content

    content = '# foo\\\n' \
              'SPAMDOPTIONS="--ssl-version tlsv1 -d --ssl"\n'  # still a comment
    rewritten = spamassassinconfigupdate_spamd._rewrite_spamd_option(content, ops)
    assert rewritten == content


def test_rewrite_spamd_option_last_assignment_takes_effect():
    ops = [spamassassinconfigupdate_spamd._drop_ssl_version, spamassassinconfigupdate_spamd._drop_daemonize_option]

    # Only the last assignment to a variable takes effect, so that's the only
    # assignment that we need to take care of
    content = 'SPAMDOPTIONS="-c -d"\n' \
              'SPAMDOPTIONS="-c -d -H"\n'
    rewritten = spamassassinconfigupdate_spamd._rewrite_spamd_option(content, ops)
    assert rewritten == '%s\nSPAMDOPTIONS="-c  -H"\n' % content.split('\n')[0]

    content = 'SPAMDOPTIONS="--ssl-version tlsv1 --ssl"\n' \
              'SPAMDOPTIONS="-c --ssl --ssl-version sslv3 -H"\n'
    rewritten = spamassassinconfigupdate_spamd._rewrite_spamd_option(content, ops)
    assert rewritten == '%s\nSPAMDOPTIONS="-c --ssl  -H"\n' % content.split('\n')[0]


def test_rewrite_spamd_option_multiline_value():
    ops = [spamassassinconfigupdate_spamd._drop_ssl_version, spamassassinconfigupdate_spamd._drop_daemonize_option]
    content = 'SPAMDOPTIONS="--ssl \\\n--ssl-version sslv3"\n'
    rewritten = spamassassinconfigupdate_spamd._rewrite_spamd_option(content, ops)
    assert rewritten == 'SPAMDOPTIONS="--ssl "\n'


def test_rewrite_spamd_config():
    facts = SpamassassinFacts(spamd_ssl_version='tlsv1', service_overriden=False)
    content = '# Options passed to spamd\n' \
              'SPAMDOPTIONS="-c -d -m5 --ssl -H --ssl-version tlsv1"\n'

    rewritten = spamassassinconfigupdate_spamd._rewrite_spamd_config(facts, content)

    assert rewritten == '# Options passed to spamd\n' \
                        'SPAMDOPTIONS="-c  -m5 --ssl -H "\n'


def test_rewrite_spamd_config_service_overriden():
    # If the service is overridden, the service type (simple/forking) remains
    # the same after upgrade. So we must not remove the -d option.
    facts = SpamassassinFacts(spamd_ssl_version='sslv3', service_overriden=True)
    content = '# Options to spamd\n' \
              'SPAMDOPTIONS="-c -d -m5 --ssl -H --ssl-version sslv3"\n'

    rewritten = spamassassinconfigupdate_spamd._rewrite_spamd_config(facts, content)

    assert rewritten == '# Options to spamd\n' \
                        'SPAMDOPTIONS="-c -d -m5 --ssl -H "\n'


class MockBackup(object):
    def __init__(self, to_raise=None):
        self.to_raise = to_raise
        self.called = 0
        self.paths = []

    def __call__(self, path):
        self.called += 1
        self.paths.append(path)
        if self.to_raise:
            raise self.to_raise
        return '/path/to/backup'


class MockFileOperations(object):
    def __init__(self, read_error=None, write_error=None):
        self.files = {}
        self.files_read = {}
        self.files_written = {}
        self.read_called = 0
        self.write_called = 0
        self.read_error = read_error
        self.write_error = write_error

    def _increment_read_counters(self, path):
        self.read_called += 1
        self.files_read.setdefault(path, 0)
        self.files_read[path] += 1

    def read(self, path):
        self._increment_read_counters(path)
        if self.read_error:
            raise self.read_error
        return self.files[path]

    def _increment_write_counters(self, path):
        self.write_called += 1
        self.files_written.setdefault(path, 0)
        self.files_written[path] += 1

    def write(self, path, content):
        self._increment_write_counters(path)
        if self.write_error:
            raise self.write_error
        self.files[path] = content


def test_migrate_spamd_config():
    facts = SpamassassinFacts(service_overriden=False, spamd_ssl_version='tlsv1')
    fileops = MockFileOperations()
    content = ('# foo\n' +
               SYSCONFIG_VARIABLE + '="-c --ssl-version tlsv1 -hdx"\n' +
               '# bar \n')
    fileops.files[SYSCONFIG_SPAMASSASSIN] = content
    backup_func = MockBackup()

    spamassassinconfigupdate_spamd.migrate_spamd_config(facts, fileops, backup_func)

    assert backup_func.called == 1
    assert backup_func.paths[0] == SYSCONFIG_SPAMASSASSIN
    expected_content = ('# foo\n' +
                        SYSCONFIG_VARIABLE + '="-c --ssl -hx"\n' +
                        '# bar \n')
    assert fileops.files[SYSCONFIG_SPAMASSASSIN] == expected_content
    assert fileops.read_called == 1
    assert fileops.files_read[SYSCONFIG_SPAMASSASSIN] == 1
    assert fileops.write_called == 1
    assert fileops.files_written[SYSCONFIG_SPAMASSASSIN] == 1


def test_migrate_spamd_config_nothing_to_migrate():
    facts = SpamassassinFacts(service_overriden=True, spamd_ssl_version=None)
    fileops = MockFileOperations()
    content = ('# foo\n' +
               SYSCONFIG_VARIABLE + '="-c -hdx"\n' +
               '# bar \n')
    fileops.files[SYSCONFIG_SPAMASSASSIN] = content
    backup_func = MockBackup()

    spamassassinconfigupdate_spamd.migrate_spamd_config(facts, fileops, backup_func)

    assert fileops.read_called == 0
    assert fileops.write_called == 0
    assert backup_func.called == 0


def test_migrate_spamd_config_no_write_if_backup_fails():
    # OSError (e.g. os.open)
    facts = SpamassassinFacts(service_overriden=False, spamd_ssl_version='tlsv1')
    fileops = MockFileOperations()
    content = ('# foo\n' +
               SYSCONFIG_VARIABLE + '="-c --ssl-version tlsv1 -hdx"\n' +
               '# bar \n')
    fileops.files[SYSCONFIG_SPAMASSASSIN] = content
    backup_func = MockBackup(to_raise=make_OSError(errno.EACCES))

    spamassassinconfigupdate_spamd.migrate_spamd_config(facts, fileops, backup_func)

    assert backup_func.called == 1
    assert fileops.write_called == 0

    # IOError (e.g. file.read)
    facts = SpamassassinFacts(service_overriden=False, spamd_ssl_version='tlsv1')
    fileops = MockFileOperations()
    content = ('# foo\n' +
               SYSCONFIG_VARIABLE + '="-c --ssl-version tlsv1 -hdx"\n' +
               '# bar \n')
    fileops.files[SYSCONFIG_SPAMASSASSIN] = content
    backup_func = MockBackup(to_raise=make_IOError(errno.EACCES))

    spamassassinconfigupdate_spamd.migrate_spamd_config(facts, fileops, backup_func)

    assert backup_func.called == 1
    assert fileops.write_called == 0


def test_migrate_spamd_config_read_failure():
    # OSError (e.g. os.open)
    facts = SpamassassinFacts(service_overriden=False, spamd_ssl_version='tlsv1')
    fileops = MockFileOperations(read_error=make_OSError(errno.EACCES))
    content = ('# foo\n' +
               SYSCONFIG_VARIABLE + '="-c --ssl-version tlsv1 -hdx"\n' +
               '# bar \n')
    fileops.files[SYSCONFIG_SPAMASSASSIN] = content
    backup_func = MockBackup()

    spamassassinconfigupdate_spamd.migrate_spamd_config(facts, fileops, backup_func)

    # The main purpose of this test is to check that exceptions are handled
    # properly. The following assertions are supplementary.
    assert fileops.read_called == 1
    assert fileops.write_called == 0

    # IOError (e.g. builtin open)
    facts = SpamassassinFacts(service_overriden=False, spamd_ssl_version='tlsv1')
    fileops = MockFileOperations(read_error=make_IOError(errno.EACCES))
    content = ('# foo\n' +
               SYSCONFIG_VARIABLE + '="-c --ssl-version tlsv1 -hdx"\n' +
               '# bar \n')
    fileops.files[SYSCONFIG_SPAMASSASSIN] = content
    backup_func = MockBackup()

    spamassassinconfigupdate_spamd.migrate_spamd_config(facts, fileops, backup_func)

    assert fileops.read_called == 1
    assert fileops.write_called == 0


def test_migrate_spamd_config_write_failure():
    # OSError (e.g. os.open)
    facts = SpamassassinFacts(service_overriden=False, spamd_ssl_version='tlsv1')
    fileops = MockFileOperations(write_error=make_OSError(errno.EACCES))
    content = ('# foo\n' +
               SYSCONFIG_VARIABLE + '="-c --ssl-version tlsv1 -hdx"\n' +
               '# bar \n')
    fileops.files[SYSCONFIG_SPAMASSASSIN] = content
    backup_func = MockBackup()

    spamassassinconfigupdate_spamd.migrate_spamd_config(facts, fileops, backup_func)

    # The main purpose of this test is to check that exceptions are handled
    # properly. The following assertions are supplementary.
    assert fileops.read_called == 1
    assert fileops.write_called == 1

    # IOError (e.g. builtin open)
    facts = SpamassassinFacts(service_overriden=False, spamd_ssl_version='tlsv1')
    fileops = MockFileOperations(write_error=make_IOError(errno.EACCES))
    content = ('# foo\n' +
               SYSCONFIG_VARIABLE + '="-c --ssl-version tlsv1 -hdx"\n' +
               '# bar \n')
    fileops.files[SYSCONFIG_SPAMASSASSIN] = content
    backup_func = MockBackup()

    spamassassinconfigupdate_spamd.migrate_spamd_config(facts, fileops, backup_func)

    assert fileops.read_called == 1
    assert fileops.write_called == 1


def test_migrate_spamd_config_no_writes_with_unmodified_config():
    # Test that no writes are performed if the sysconfig file is in its
    # default state and thus gets replaced during the RPM upgrade.
    facts = SpamassassinFacts(service_overriden=False, spamd_ssl_version='tlsv1')
    # Content of the sysconfig file from RHEL-8
    content = ('# Options to spamd\n'
               'SPAMDOPTIONS="-c -m5 -H --razor-home-dir=\'/var/lib/razor/\' --razor-log-file=\'sys-syslog\'"\n')
    fileops = MockFileOperations()
    fileops.files[SYSCONFIG_SPAMASSASSIN] = content
    backup_func = MockBackup()

    spamassassinconfigupdate_spamd.migrate_spamd_config(facts, fileops, backup_func)

    assert backup_func.called == 0
    assert fileops.read_called == 1
    assert fileops.write_called == 0



import os
import shutil

import pytest

from leapp.exceptions import StopActorExecutionError
from leapp.libraries.actor import upgradeinitramfsgenerator
from leapp.libraries.common.config import architecture
from leapp.libraries.common.testutils import CurrentActorMocked, logger_mocked, produce_mocked
from leapp.utils.deprecation import suppress_deprecation

from leapp.models import (  # isort:skip
    FIPSInfo,
    RequiredUpgradeInitramPackages,  # deprecated
    UpgradeDracutModule,  # deprecated
    BootContent,
    CopyFile,
    DracutModule,
    KernelModule,
    TargetUserSpaceUpgradeTasks,
    UpgradeInitramfsTasks,
)

CUR_DIR = os.path.dirname(os.path.abspath(__file__))
PKGS = ['pkg{}'.format(c) for c in 'ABCDEFGHIJ']
FILES = [
    CopyFile(src='/host/srcfile{}'.format(i), dst='/cont/dstfile{}'.format(i))
    for i in range(5)
]
MODULES = [
    ('moduleA', None),
    ('moduleB', None),
    ('moduleC', '/some/path/moduleC'),
    ('moduleD', '/some/path/moduleD'),
]


@pytest.fixture
def adjust_cwd():
    previous_cwd = os.getcwd()
    os.chdir(os.path.join(CUR_DIR, "../"))
    yield
    os.chdir(previous_cwd)


def _ensure_list(data):
    return data if isinstance(data, list) else [data]


def gen_TUSU(packages, copy_files=None):
    packages = _ensure_list(packages)

    if not copy_files:
        copy_files = []
    copy_files = _ensure_list(copy_files)

    return TargetUserSpaceUpgradeTasks(install_rpms=packages, copy_files=copy_files)


@suppress_deprecation(RequiredUpgradeInitramPackages)
def gen_RUIP(packages):
    packages = _ensure_list(packages)
    return RequiredUpgradeInitramPackages(packages=packages)


def gen_UIT(dracut_modules, kernel_modules, files):
    files = _ensure_list(files)

    dracut_modules = [DracutModule(name=i[0], module_path=i[1]) for i in _ensure_list(dracut_modules)]
    kernel_modules = [KernelModule(name=i[0], module_path=i[1]) for i in _ensure_list(kernel_modules)]

    return UpgradeInitramfsTasks(include_files=files,
                                 include_dracut_modules=dracut_modules,
                                 include_kernel_modules=kernel_modules,
                                 )


@suppress_deprecation(UpgradeDracutModule)
def gen_UDM_list(data):
    if not isinstance(data, list):
        data = [data]
    return [UpgradeDracutModule(name=i[0], module_path=i[1]) for i in data]


class MockedContext(object):
    def __init__(self):
        self.called_copy_from = []
        self.called_copytree_from = []
        self.called_copy_to = []
        self.called_call = []
        self.called_makedirs = []
        self.content = set()
        self.base_dir = "/base/dir"
        """
        Content (paths) that should exists regarding the used methods.

        It's not 100% same. Just dst paths are copied here. Ignoring differences
        between copy to /path/to/filename and /path/to/dirname which in real
        world could be different. For our purposes it's ok as it is now.

        Point is, that in case of use context.remove_tree(), we are able to
        detect whether something what is expected to be present is not missing.
        """

    def copy_from(self, src, dst):
        self.called_copy_from.append((src, dst))

    def copytree_from(self, src, dst):
        self.called_copytree_from.append((src, dst))

    def copy_to(self, src, dst):
        self.called_copy_to.append((src, dst))
        self.content.add(dst)

    def copytree_to(self, src, dst):
        self.called_copy_to.append((src, dst))
        self.content.add(dst)

    def makedirs(self, path):
        self.called_makedirs.append(path)

    def remove_tree(self, path):
        # make list for iteration as change of the set is expected during the
        # iteration, which could lead to runtime error
        for item in list(self.content):
            # ensure the / is the last character to simulate dirname
            dir_fmt_path = path if path[-1] == '/' else path + '/'
            if item == path or item.startswith(dir_fmt_path):
                # remove the file or everything inside dir (including dir)
                self.content.remove(item)

    def call(self, *args, **kwargs):
        self.called_call.append((args, kwargs))

    def full_path(self, path):
        return os.path.join(self.base_dir, os.path.abspath(path).lstrip('/'))


class MockedLogger(logger_mocked):

    def error(self, *args, **dummy):
        self.errmsg.extend(args)


@pytest.mark.parametrize('arch', architecture.ARCH_SUPPORTED)
def test_copy_boot_files(monkeypatch, arch):
    kernel = 'vmlinuz-upgrade.{}'.format(arch)
    kernel_hmac = '.vmlinuz-upgrade.{}.hmac'.format(arch)
    initram = 'initramfs-upgrade.{}.img'.format(arch)
    bootc = BootContent(
        kernel_path=os.path.join('/boot', kernel),
        kernel_hmac_path=os.path.join('/boot', kernel_hmac),
        initram_path=os.path.join('/boot', initram)
    )

    context = MockedContext()
    monkeypatch.setattr(upgradeinitramfsgenerator.api, 'current_actor', CurrentActorMocked(arch=arch))
    monkeypatch.setattr(upgradeinitramfsgenerator.api, 'produce', produce_mocked())

    def create_upgrade_hmac_from_target_hmac_mock(original_hmac_path, upgrade_hmac_path, upgrade_kernel):
        hmac_file = '.{}.hmac'.format(upgrade_kernel)
        assert original_hmac_path == os.path.join(context.full_path('/artifacts'), hmac_file)
        assert upgrade_hmac_path == bootc.kernel_hmac_path

    monkeypatch.setattr(upgradeinitramfsgenerator,
                        'create_upgrade_hmac_from_target_hmac',
                        create_upgrade_hmac_from_target_hmac_mock)

    upgradeinitramfsgenerator.copy_boot_files(context)
    assert len(context.called_copy_from) == 2
    assert (os.path.join('/artifacts', kernel), bootc.kernel_path) in context.called_copy_from
    assert (os.path.join('/artifacts', initram), bootc.initram_path) in context.called_copy_from

    assert upgradeinitramfsgenerator.api.produce.called == 1
    assert upgradeinitramfsgenerator.api.produce.model_instances[0] == bootc


class MockedCopyArgs(object):
    def __init__(self):
        self.args = None

    def __call__(self, *args):
        self.args = args


def _sort_files(copy_files):
    return sorted(copy_files, key=lambda x: (x.src, x.dst))


def test_prepare_userspace_for_initram_no_script(monkeypatch):
    monkeypatch.setattr(upgradeinitramfsgenerator.api, 'get_actor_file_path', lambda dummy: None)
    with pytest.raises(StopActorExecutionError) as err:
        upgradeinitramfsgenerator.prepare_userspace_for_initram(MockedContext())
    assert err.value.message == 'Mandatory script to generate initram not available.'


@pytest.mark.parametrize('input_msgs,pkgs,files', [
    # deprecated packages only, without files -- original functionality
    ([gen_RUIP([])], [], []),
    ([gen_RUIP(PKGS[0])], PKGS[0], []),
    ([gen_RUIP(PKGS)], PKGS, []),

    # packages only, without files -- new analogy to previous functionality
    ([gen_TUSU([])], [], []),
    ([gen_TUSU(PKGS[0])], PKGS[0], []),
    ([gen_TUSU(PKGS)], PKGS, []),

    # packages only, mix of deprecated and new models - same sets
    ([gen_RUIP([]), gen_TUSU([])], [], []),
    ([gen_RUIP(PKGS[0]), gen_TUSU(PKGS[0])], PKGS[0], []),
    ([gen_RUIP(PKGS), gen_TUSU(PKGS)], PKGS, []),

    # packages only, mix of deprecated and new models - disjoint sets
    ([gen_RUIP(PKGS[0]), gen_TUSU(PKGS[1])], PKGS[0:2], []),
    ([gen_RUIP([]), gen_TUSU(PKGS)], PKGS, []),
    ([gen_RUIP(PKGS), gen_TUSU([])], PKGS, []),
    ([gen_RUIP(PKGS[0:5]), gen_TUSU(PKGS[5:])], PKGS, []),

    # packages only, mix of deprecated and new models - mixed
    ([gen_RUIP(PKGS[0:7]), gen_TUSU(PKGS[5:])], PKGS, []),

    # files only
    ([gen_TUSU([], FILES[0])], [], FILES[0]),
    ([gen_TUSU([], FILES)], [], FILES),

    # packages and files
    ([gen_RUIP(PKGS[0]), gen_TUSU(PKGS[1], FILES)], PKGS[0:2], FILES),
    ([gen_RUIP(PKGS[0:7]), gen_TUSU(PKGS[5:], FILES)], PKGS, FILES),
])
def test_prepare_userspace_for_initram(monkeypatch, adjust_cwd, input_msgs, pkgs, files):
    monkeypatch.setattr(upgradeinitramfsgenerator.api, 'current_actor', CurrentActorMocked(msgs=input_msgs))
    monkeypatch.setattr(upgradeinitramfsgenerator, '_install_initram_deps', MockedCopyArgs())
    monkeypatch.setattr(upgradeinitramfsgenerator, '_copy_files', MockedCopyArgs())

    context = MockedContext()
    upgradeinitramfsgenerator.prepare_userspace_for_initram(context)

    # check the upgradeinitramfsgenerator script is copied into the container
    initram_copy = (
        upgradeinitramfsgenerator.api.get_actor_file_path(upgradeinitramfsgenerator.INITRAM_GEN_SCRIPT_NAME),
        os.path.join('/', upgradeinitramfsgenerator.INITRAM_GEN_SCRIPT_NAME)
    )
    assert initram_copy in context.called_copy_to

    # check the set of packages required to be installed matches expectations
    _pkgs = set(pkgs) if isinstance(pkgs, list) else set([pkgs])
    assert upgradeinitramfsgenerator._install_initram_deps.args[0] == _pkgs

    # check the set of files to be copied into the container matches exp
    _files = _sort_files(files) if isinstance(files, list) else [files]
    assert _sort_files(upgradeinitramfsgenerator._copy_files.args[1]) == _files


class MockedGetFspace(object):
    def __init__(self, space):
        self.space = space

    def __call__(self, dummy_path, convert_to_mibs=False):
        if not convert_to_mibs:
            return self.space
        return int(self.space / 1024 / 1024)  # noqa: W1619; pylint: disable=old-division


@pytest.mark.parametrize('input_msgs,dracut_modules,kernel_modules', [
    # test dracut modules with UpgradeDracutModule(s) - orig functionality
    (gen_UDM_list(MODULES[0]), MODULES[0], []),
    (gen_UDM_list(MODULES), MODULES, []),

    # test dracut modules with UpgradeInitramfsTasks - new functionality
    ([gen_UIT(MODULES[0], MODULES[0], [])], MODULES[0], MODULES[0]),
    ([gen_UIT(MODULES, MODULES, [])], MODULES, MODULES),

    # test dracut modules with old and new models
    (gen_UDM_list(MODULES[1]) + [gen_UIT(MODULES[2], [], [])], MODULES[1:3], []),
    (gen_UDM_list(MODULES[2:]) + [gen_UIT(MODULES[0:2], [], [])], MODULES, []),
    (gen_UDM_list(MODULES[1]) + [gen_UIT([], MODULES[2], [])], MODULES[1], MODULES[2]),
    (gen_UDM_list(MODULES[2:]) + [gen_UIT([], MODULES[0:2], [])], MODULES[2:], MODULES[0:2]),

    # TODO(pstodulk): test include files missing (relates #376)
])
def test_generate_initram_disk(monkeypatch, input_msgs, dracut_modules, kernel_modules):
    context = MockedContext()
    curr_actor = CurrentActorMocked(msgs=input_msgs, arch=architecture.ARCH_X86_64)
    monkeypatch.setattr(upgradeinitramfsgenerator.api, 'current_actor', curr_actor)
    monkeypatch.setattr(upgradeinitramfsgenerator, 'copy_dracut_modules', MockedCopyArgs())
    monkeypatch.setattr(upgradeinitramfsgenerator, '_get_target_kernel_version', lambda _: '')
    monkeypatch.setattr(upgradeinitramfsgenerator, 'copy_kernel_modules', MockedCopyArgs())
    monkeypatch.setattr(upgradeinitramfsgenerator, 'copy_boot_files', lambda dummy: None)
    monkeypatch.setattr(upgradeinitramfsgenerator, '_get_fspace', MockedGetFspace(2*2**30))
    upgradeinitramfsgenerator.generate_initram_disk(context)

    # TODO(pstodulk): add tests for the check of the free space (sep. from this func)

    # test now just that all modules have been passed for copying - so we know
    # all modules have been consumed
    detected_dracut_modules = set()
    _dracut_modules = set(dracut_modules) if isinstance(dracut_modules, list) else set([dracut_modules])
    for dracut_module in upgradeinitramfsgenerator.copy_dracut_modules.args[1]:
        module = (dracut_module.name, dracut_module.module_path)
        assert module in _dracut_modules
        detected_dracut_modules.add(module)
    assert detected_dracut_modules == _dracut_modules

    detected_kernel_modules = set()
    _kernel_modules = set(kernel_modules) if isinstance(kernel_modules, list) else set([kernel_modules])
    for kernel_module in upgradeinitramfsgenerator.copy_kernel_modules.args[1]:
        module = (kernel_module.name, kernel_module.module_path)
        assert module in _kernel_modules
        detected_kernel_modules.add(module)
    assert detected_kernel_modules == _kernel_modules

    # TODO(pstodulk): this test is not created properly, as context.call check
    # is skipped completely. Testing will more convenient with fixed #376
    # similar to the files...


def test_copy_dracut_modules_rmtree_ignore(monkeypatch):
    context = MockedContext()

    def raise_env_error(dummy):
        raise EnvironmentError('an error')

    def mock_context_path_exists(path):
        full_path_content = {context.full_path(i) for i in context.content}
        return full_path_content.intersection(set([path, path + '/'])) != set()

    monkeypatch.setattr(os.path, 'exists', mock_context_path_exists)
    dmodules = [DracutModule(name='foo', module_path='/path/foo')]
    upgradeinitramfsgenerator.copy_dracut_modules(context, dmodules)
    assert context.content

    # env error should be ignored in this case
    context.content = set()
    context.remove_tree = raise_env_error
    upgradeinitramfsgenerator.copy_dracut_modules(context, dmodules)
    assert context.content


@pytest.mark.parametrize('kind', ['dracut', 'kernel'])
def test_copy_modules_fail(monkeypatch, kind):
    context = MockedContext()

    def copytree_to_error(src, dst):
        raise shutil.Error('myerror: {}, {}'.format(src, dst))

    def mock_context_path_exists(path):
        full_path_content = {context.full_path(i) for i in context.content}
        return full_path_content.intersection(set([path, path + '/'])) != set()

    context.copytree_to = copytree_to_error
    monkeypatch.setattr(os.path, 'exists', mock_context_path_exists)
    monkeypatch.setattr(upgradeinitramfsgenerator.api, 'current_logger', MockedLogger())
    monkeypatch.setattr(upgradeinitramfsgenerator, '_get_target_kernel_modules_dir', lambda _: '/kernel_modules')

    module_class = None
    copy_fn = None
    if kind == 'dracut':
        module_class = DracutModule
        copy_fn = upgradeinitramfsgenerator.copy_dracut_modules
        dst_path = 'dracut'
    elif kind == 'kernel':
        module_class = KernelModule
        copy_fn = upgradeinitramfsgenerator.copy_kernel_modules
        dst_path = 'kernel_modules'

    modules = [module_class(name='foo', module_path='/path/foo')]
    with pytest.raises(StopActorExecutionError) as err:
        copy_fn(context, modules)
    assert err.value.message.startswith('Failed to install {kind} modules'.format(kind=kind))
    expected_err_log = 'Failed to copy {kind} module "foo" from "/path/foo" to "/base/dir/{dst_path}"'.format(
            kind=kind, dst_path=dst_path)
    assert expected_err_log in upgradeinitramfsgenerator.api.current_logger.errmsg


@pytest.mark.parametrize('kind', ['dracut', 'kernel'])
def test_copy_modules_duplicate_skip(monkeypatch, kind):
    context = MockedContext()

    def mock_context_path_exists(path):
        full_path_content = {context.full_path(i) for i in context.content}
        return full_path_content.intersection(set([path, path + '/'])) != set()

    monkeypatch.setattr(os.path, 'exists', mock_context_path_exists)
    monkeypatch.setattr(upgradeinitramfsgenerator.api, 'current_logger', MockedLogger())
    monkeypatch.setattr(upgradeinitramfsgenerator, '_get_target_kernel_modules_dir', lambda _: '/kernel_modules')

    module_class = None
    copy_fn = None
    if kind == 'dracut':
        module_class = DracutModule
        copy_fn = upgradeinitramfsgenerator.copy_dracut_modules
    elif kind == 'kernel':
        module_class = KernelModule
        copy_fn = upgradeinitramfsgenerator.copy_kernel_modules

    module = module_class(name='foo', module_path='/path/foo')
    modules = [module, module]

    copy_fn(context, modules)

    debugmsg = 'The foo {kind} module has been already installed. Skipping.'.format(kind=kind)
    assert context.content
    assert len(context.called_copy_to) == 1
    assert debugmsg in upgradeinitramfsgenerator.api.current_logger.dbgmsg



from leapp import reporting
from leapp.libraries.actor.satellite_upgrade_check import satellite_upgrade_check
from leapp.libraries.common.testutils import create_report_mocked
from leapp.models import SatelliteFacts, SatellitePostgresqlFacts


def test_old_data(monkeypatch):
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())

    satellite_upgrade_check(SatelliteFacts(has_foreman=True,
                            postgresql=SatellitePostgresqlFacts(local_postgresql=True, old_var_lib_pgsql_data=True)))

    assert reporting.create_report.called == 2

    expected_title = 'Old PostgreSQL data found in /var/lib/pgsql/data'
    assert next((report for report in reporting.create_report.reports if report.get('title') == expected_title), None)

    expected_title = 'Satellite PostgreSQL data migration'
    assert next((report for report in reporting.create_report.reports if report.get('title') == expected_title), None)


def test_no_old_data(monkeypatch):
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())

    satellite_upgrade_check(SatelliteFacts(has_foreman=True,
                            postgresql=SatellitePostgresqlFacts(local_postgresql=True, old_var_lib_pgsql_data=False)))

    assert reporting.create_report.called == 1

    expected_title = 'Satellite PostgreSQL data migration'

    assert expected_title == reporting.create_report.report_fields['title']


def test_same_disk(monkeypatch):
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())

    satellite_upgrade_check(SatelliteFacts(has_foreman=True,
                            postgresql=SatellitePostgresqlFacts(local_postgresql=True, same_partition=True)))

    assert reporting.create_report.called == 1

    expected_title = 'Satellite PostgreSQL data migration'
    expected_summary = 'Your PostgreSQL data will be automatically migrated.'
    expected_reindex = 'all databases will require a REINDEX'

    assert expected_title == reporting.create_report.report_fields['title']
    assert expected_summary in reporting.create_report.report_fields['summary']
    assert expected_reindex in reporting.create_report.report_fields['summary']


def test_different_disk_sufficient_storage(monkeypatch):
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())

    satellite_upgrade_check(SatelliteFacts(has_foreman=True,
                            postgresql=SatellitePostgresqlFacts(local_postgresql=True, same_partition=False,
                                                                space_required=5, space_available=10)))

    assert reporting.create_report.called == 1

    expected_title = 'Satellite PostgreSQL data migration'
    expected_summary = 'You currently have enough free storage to move the data'
    expected_reindex = 'all databases will require a REINDEX'

    assert expected_title == reporting.create_report.report_fields['title']
    assert expected_summary in reporting.create_report.report_fields['summary']
    assert expected_reindex in reporting.create_report.report_fields['summary']


def test_different_disk_insufficient_storage(monkeypatch):
    monkeypatch.setattr(reporting, 'create_report', create_report_mocked())

    satellite_upgrade_check(SatelliteFacts(has_foreman=True,
                            postgresql=SatellitePostgresqlFacts(local_postgresql=True, same_partition=False,
                                                                space_required=10, space_available=5)))

    assert reporting.create_report.called == 1

    expected_title = 'Satellite PostgreSQL data migration'
    expected_summary = "You currently don't have enough free storage to move the data"

    assert expected_title == reporting.create_report.report_fields['title']
    assert expected_summary in reporting.create_report.report_fields['summary']



import pytest

from leapp.libraries import stdlib
from leapp.libraries.actor import setsystemdservicesstate
from leapp.libraries.common import systemd
from leapp.libraries.common.testutils import CurrentActorMocked, logger_mocked
from leapp.libraries.stdlib import api, CalledProcessError
from leapp.models import SystemdServicesTasks


class MockedSystemdCmd(object):
    def __init__(self):
        self.units = []

    def __call__(self, unit, *args, **kwargs):
        self.units.append(unit)
        return {}


@pytest.mark.parametrize(
    ('msgs', 'expect_enable_units', 'expect_disable_units'),
    [
        (
            [SystemdServicesTasks(to_enable=['hello.service'],
                                  to_disable=['getty.service'])],
            ['hello.service'],
            ['getty.service']
        ),
        (
            [SystemdServicesTasks(to_disable=['getty.service'])],
            [],
            ['getty.service']
        ),
        (
            [SystemdServicesTasks(to_enable=['hello.service'])],
            ['hello.service'],
            []
        ),
        (
            [SystemdServicesTasks()],
            [],
            []
        ),
    ]
)
def test_process(monkeypatch, msgs, expect_enable_units, expect_disable_units):
    mocked_enable = MockedSystemdCmd()
    monkeypatch.setattr(systemd, 'enable_unit', mocked_enable)

    mocked_disable = MockedSystemdCmd()
    monkeypatch.setattr(systemd, 'disable_unit', mocked_disable)

    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=msgs))

    setsystemdservicesstate.process()

    assert mocked_enable.units == expect_enable_units
    assert mocked_disable.units == expect_disable_units


def test_process_invalid(monkeypatch):

    def mocked_run(cmd, *args, **kwargs):
        if cmd == ['systemctl', 'enable', 'invalid.service']:
            message = 'Command {0} failed with exit code {1}.'.format(str(cmd), 1)
            raise CalledProcessError(message, cmd, 1)

    msgs = [SystemdServicesTasks(to_enable=['invalid.service'])]

    monkeypatch.setattr(systemd, 'run', mocked_run)
    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=msgs))
    monkeypatch.setattr(api, 'current_logger', logger_mocked())

    setsystemdservicesstate.process()

    expect_msg = ("Failed to enable systemd unit \"invalid.service\". Message:"
                  " Command ['systemctl', 'enable', 'invalid.service'] failed with exit code 1.")
    assert expect_msg in api.current_logger.errmsg


def test_enable_disable_conflict_logged(monkeypatch):
    msgs = [SystemdServicesTasks(to_enable=['hello.service'], to_disable=['hello.service'])]

    mocked_enable = MockedSystemdCmd()
    monkeypatch.setattr(systemd, 'enable_unit', mocked_enable)

    mocked_disable = MockedSystemdCmd()
    monkeypatch.setattr(systemd, 'disable_unit', mocked_disable)

    monkeypatch.setattr(api, 'current_actor', CurrentActorMocked(msgs=msgs))
    monkeypatch.setattr(api, 'current_logger', logger_mocked())

    setsystemdservicesstate.process()

    expect_msg = ('Attempted to both enable and disable systemd service "hello.service",'
                  ' service will be disabled.')
    assert expect_msg in api.current_logger.errmsg



from leapp.actors import Actor
from leapp.libraries.actor import cupsfiltersmigrate
from leapp.models import DistributionSignedRPM
from leapp.tags import ApplicationsPhaseTag, IPUWorkflowTag


class CupsfiltersMigrate(Actor):
    """
    Actor for migrating package cups-filters.

    Migrating cups-filters package means adding two directives into
    /etc/cups/cups-browsed.conf - LocalQueueNamingRemoteCUPS and
    CreateIPPPrinterQueues.

    LocalQueueNamingRemoteCUPS directive indicates what will be used as a name
    for local print queue creation - the default is DNS-SD ID of advertised
    print queue now, it was the name of remote print queue in the past.

    CreateIPPPrinterQueues directive serves for telling cups-browsed to create
    local print queues for all available IPP printers.
    """

    name = 'cupsfilters_migrate'
    consumes = (DistributionSignedRPM,)
    produces = ()
    tags = (ApplicationsPhaseTag, IPUWorkflowTag)

    def process(self):
        cupsfiltersmigrate.update_cups_browsed()
